/*
 * Brimstone
 * Copyright © 2009 Adrian Williams - All rights reserved
 */

const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"] .getService(Components.interfaces.nsIClipboardHelper);

String.prototype.replaceWhiteSpace = function() {
	return this.replace(new RegExp("\\xA0", "g"), " ");
}

String.prototype.upperCaseFirst = function() {
	return this.substring(0,1).toUpperCase() + this.substring(1);
}

function convertToNumber(value) {
	if (value == "TONS") {
		return value;
	} else {
		value = value.replace(/,/g,"");
		value = value.replace(/K/gi,"000");
		return parseInt(value);
	}
}

function parseText(text, parseHints, object) {
	for (var i=0, l=parseHints.length; i<l; i++) {
		var hint = parseHints[i]
		var match = hint.re.exec(text);
		if (match) {
			var value = match[1];
			if (hint.attributeType == "number") {
				value = convertToNumber(value);
			}
			object[hint.attributeName] = value;
		}
	}
}

function getTimeStamp(offset) {
	var d = new Date();
	if (offset) {
		d.setDate(d.getDate() + offset);
	}
	return "{0:0000}-{1:00}-{2:00} {3:00}:{4:00}:{5:00}".format(d.getFullYear(), d.getMonth() + 1, d.getDate(), d.getHours(), d.getMinutes(), d.getSeconds());	
}

const STATE_START = Components.interfaces.nsIWebProgressListener.STATE_START;
const STATE_REDIRECTING = Components.interfaces.nsIWebProgressListener.STATE_REDIRECTING;
const STATE_STOP  = Components.interfaces.nsIWebProgressListener.STATE_STOP;

var RDF = Components
			.classes["@mozilla.org/rdf/rdf-service;1"]
			.getService(Components.interfaces.nsIRDFService);
var containerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"].
			            getService(Components.interfaces.nsIRDFContainerUtils);

var Brimstone = {
	id : "brimstone@xuluwarrior.org",
	installedPath : null,
	parentWindow : null,
	parentBrowser : null,
	player : null,
	players : {},
	events : [],
	eventsById : {},
	nextEventID : 1,
	alliances : {},
	parameters : {},
	lastPageName : null,
	
	onLoad : function(){
		setTimeout(Brimstone.initialise, 10);
	},
	
	browserLoad : function(event) {
		try {
			if (event.originalTarget) {
				
				var uri = event.originalTarget.documentURI;
				var match = /\w+:\/\/([^\/]+)(\/.+)/.exec(uri);
				if (match) {
					var host = match[1];
					var path = match[2];
					if (host && BrimstoneHandler.handledURLs[host]) {
						if (BrimstoneOptions.warbookHost != host) {
							BrimstoneOptions.warbookHost = host;
							Brimstone.playerChanged();
						}
						if (event.originalTarget.defaultView.frameElement && !BrimstoneOptions.leaveWarbookInFrameOption) {
							event.currentTarget.loadURI(uri, null, null);
						} else {
							BrimstoneHandler.handlePage(host, path, event.originalTarget.defaultView);
						}
						BrimstoneBrowser.renameTab(uri, Brimstone.lastPageName);
			//			BrimstoneHandler.handlePage(path, event.originalTarget.defaultView);
					} else {
						if (event.originalTarget.defaultView.frameElement) {
							var uri = event.currentTarget.currentURI.spec;
							var match = /\w+:\/\/([^\/]+)(\/.+)/.exec(uri);
							if (match) {
								var host = match[1];
								if (host && BrimstoneHandler.handledURLs[host]) {
									// This is a sub-frame of a Warbook page
									// Most likely adverts
									var links = event.originalTarget.defaultView.document.links
//									jsdump("host: " + host);
									for (var i=0, l= links.length; i < l; i++) {
//										jsdump("Adding event listener to " + links[i]);
//										links[i].addEventListener("click", BrimstoneHandler.handleClick, true);			
									}
								}
							}
							var a = 1;
						}
					}
				}
			}
		} catch(e) {
			BrimstoneErrorHandler.handleError("parsing army page", e);
		}
	},
	
	initialise : function() {
		try {
			jsdump("Initialising: " + new Date());
			Brimstone.installedPath = BrimstoneVariant.Brimstone.getInstalledPath();
			if (document.title == "Brimstone") {
				Brimstone.parentWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
				                   .getInterface(Components.interfaces.nsIWebNavigation)
				                   .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
				                   .rootTreeItem
				                   .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
				                   .getInterface(Components.interfaces.nsIDOMWindow);
				//Brimstone.parentBrowser = Brimstone.parentWindow.getBrowser();
				Brimstone.players = {};
				BrimstoneData.initialise();
				// Why can't I load parameters here?
				BrimstoneUI.initialise();
				Brimstone.parameters = BrimstoneData.loadParameters(["faction"], false);
				BrimstoneHandler.initialise();
				BrimstoneBrowser.initialise();
				Brimstone.alliances = BrimstoneData.loadAlliances({returnType : "indexedByPrimaryKey"});
				BrimstoneBrowser.addEventListener("load", Brimstone.browserLoad, true);
				BrimstoneBrowser.addEventListener("beforeunload", BrimstoneHandler.beforeUnload, true);
	
				Brimstone.playerChanged();

				BrimstonePrefObsever.register();
				var file = Components.classes["@mozilla.org/file/local;1"]
			                     .createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath(BrimstoneData.dataPath);
				file.append("events.log");
				Brimstone.eventLogPath = file.path;
				jsdump("Done initialising: " + new Date());
				if (BrimstoneVariant.Brimstone.postInitialise) {
					BrimstoneVariant.Brimstone.postInitialise();
				}
			}
		} catch(e) {
			BrimstoneErrorHandler.handleError("initialising Brimstone", e);
		}
	},
	
	getFaction : function() {
		return BrimstoneHandler.handledURLs[BrimstoneOptions.warbookHost];
	},
	
	getPlayerName : function(){
		if (BrimstoneOptions.warbookHost) {
			switch (Brimstone.faction) {
				case "bebo":
					var playerName = BrimstoneOptions.beboPlayerName;
					break;
				case "facebook":
					var playerName = BrimstoneOptions.facebookPlayerName;
					break;
			}
		}
		if (!playerName) {
			var playerName = BrimstoneOptions.oldPlayerName
		}
		return playerName;
	}, 

	playerChanged : function() {
//		jsdump("Reload");
		var playerName = Brimstone.getPlayerName();
		if (!Brimstone.player || Brimstone.player.name != playerName) {
			Brimstone.player = null;
			Brimstone.players = {};
			Brimstone.events = [];
			BrimstoneUI.clear();
			Brimstone.player = BrimstoneData.loadPlayer(playerName);	
			if (Brimstone.player) {
				Brimstone.player.faction = Brimstone.faction;
				BrimstoneUI.playerChanged();
			}
		}
		if (Brimstone.faction) {
			BrimstoneHandler.factionChanged();
		}
	},
	
	cleanData : function() {
		BrimstoneData.cleanData();
		Brimstone.player = null;
		Brimstone.players = {};
	},
	
	onClose : function() {
		jsdump("On Close");
		if (document.title == "Brimstone") {
			BrimstoneUI.terminate();
			BrimstoneData.close();
			BrimstoneBrowser.removeEventListener("load", Brimstone.browserLoad, true);
			BrimstoneBrowser.onClose();
			BrimstonePrefObsever.unregister();
			// There seems to be a problem with the DB connection locking the database file and
			// taking time to release it.  If the user closes Brimstone and immediately reopens it bad things can happen
			// What to do?
		}
	},
	
	writeEventLog : function(eventText) {
//		jsdump("Request to write");
		if (BrimstoneOptions.writeEventLog) {
			eventText += "\n";
//			jsdump("Writing eventText: '" + eventText + "'");
			var file = Components.classes["@mozilla.org/file/local;1"]
		                     .createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(Brimstone.eventLogPath);
			// file is nsIFile, data is a string
			var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
			                         .createInstance(Components.interfaces.nsIFileOutputStream);
			
			// use 0x02 | 0x10 to open file for appending.
			foStream.init(file, 0x02 | 0x08 | 0x10 , BrimstoneData.filePermissions, 0); // write, create, append
			foStream.write(eventText, eventText.length);
			foStream.close();
			
		}
	},
}

Brimstone.__defineGetter__("faction", function() {
	return Brimstone.parameters["faction"].value;
});

Brimstone.__defineSetter__("faction", function(faction) {
	if (Brimstone.parameters["faction"].value != faction) {
		Brimstone.parameters["faction"].value = faction;
		Brimstone.parameters["faction"].save();
		Brimstone.playerChanged();
	}
});

var BrimstoneListener =
{
    QueryInterface: function(aIID) {
      if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
          aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
          aIID.equals(Components.interfaces.nsIXULBrowserWindow) ||
          aIID.equals(Components.interfaces.nsISupports) ||
          aIID.equals(Components.interfaces.nsIHttpNotify) ||
          aIID.equals(Components.interfaces.nsIObserver))
        return this;
      throw Components.results.NS_NOINTERFACE;
    },
  
    init: function() {
		jsdump("Init");
    },
  
    destroy: function() {
    },
  
    onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
		try {
			BrimstoneHandler.handleStateChange(aWebProgress, aRequest, aStateFlags, aStatus);		
		} catch(e) {
			BrimstoneErrorHandler.handleError("State Changed", e);
		}
    },

	onProgressChange : function(aWebProgress, aRequest, aCurSelfProgress,
                         aMaxSelfProgress,
                         aCurTotalProgress,
                         aMaxTotalProgress){
			if (aRequest.name.substring(0, 27) == "http://warbook.freewebz.com") {
				jsdump("Progress Change: "+ aRequest.name);						 	
			}
			if (aRequest.name.substring(0, 31) == "http://warbook.facebook.sgn.com") {
				jsdump("Progress Change: "+ aRequest.name);	
//				aWebProgress.DOMWindow.console.log(aWebProgress.DOMWindow.document);
				var sgn_bar = aWebProgress.DOMWindow.document.getElementById("sgn_bar");
				if (sgn_bar) {
					jsdump("Found sgn_bar");
					var parent = sgn_bar.parentNode;
					parent.removeChild(sgn_bar);
				}
			}
			
			if (aRequest.name.substring(0, 24) == "http://static.sgnbar.com") {
				jsdump("Loading http://static.sgnbar.com");
				aWebProgress.DOMWindow.stop();
			}
	},

	onLocationChange : function(aWebProgress, aRequest, aLocation){
//		BrimstoneUI.appendLog("location change");
//        Brimstone.onLocationChange(a, b, c);
    },

	onStatusChange : function(aWebProgress, aRequest, aStatus, aMessage) {},

  	onSecurityChange : function(aWebProgress, aRequest, aState){},
						
  
    
    onLinkIconAvailable: function(a) {
    }
};

var BrimstoneHandler = {
	handledURLs : {"warbook.facebook.sgn.com" : "facebook", "warbook.workisboring.com" : "facebook", "optimus.freewebz.com" : "bebo", "apps.facebook.com" : "roti", "67.202.17.250" : "roti"},
	handlers : {},
	timestamp : null,
	lastMessageLog : null,
	
	initialise : function() {
		try {
			BrimstoneHandler.handlers["warbook.facebook.sgn.com"] = BrimstoneWarbookHandler;
			BrimstoneHandler.handlers["warbook.workisboring.com"] = BrimstoneWarbookHandler;
			BrimstoneHandler.handlers["apps.facebook.com"] = BrimstoneROTIHandler;
			BrimstoneHandler.handlers["67.202.17.250"] = BrimstoneROTIHandler;
			BrimstoneHandler.parameters = BrimstoneData.loadHandlerParameters();
			BrimstoneWarbookHandler.initialise();
			BrimstoneROTIHandler.initialise();
			BrimstoneBrowser.addEventListener("click", BrimstoneHandler.handleClick, true);
		} catch(e) {
			BrimstoneErrorHandler.handleError("initialising BrimstoneHandler", e);
		}
	},
	
	factionChanged : function() {
//		jsdump("BrimstoneHandler.factionChanged");
		BrimstoneHandler.lastMessageLog = BrimstoneHandler.parameters[Brimstone.faction].lastmessagelog;
		if (BrimstoneHandler.lastMessageLog.value ==  null) {
			BrimstoneHandler.lastMessageLog.value = "";
		}
		BrimstoneWarbookHandler.factionChanged();
	},
	
	handleClick : function(e) {
		try {
//			jsdump("click: " + e.target);
//			jsdump("name: " + e.target.nodeName);
			// Was this a click on a link?
			if (e.currentTarget.nodeName == "browser"  && e.target.nodeName.toUpperCase() == "A") {
				// Was this a click on a tab 'managed' by Brimstone
				var tabHost = e.currentTarget.currentURI.host
//				jsdump("tabHost: " + tabHost);
				if (BrimstoneHandler.handlers[tabHost]) {
					// Take control
					var linkHost = e.target.host;
//					jsdump("linkHost: " + linkHost);
					if (BrimstoneHandler.handlers[linkHost]) {
						BrimstoneHandler.handlers[linkHost].handleClick(e);
					} else if (false) { // If we are using BrimstoneDesktop, then links should be sent to default browser
						e.preventDefault();
						var url = e.target.href;
						var uri = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(url, null, null);
						
						var com = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"];
						var httpHandler = com.createInstance(Components.interfaces.nsIExternalProtocolService);
						
						httpHandler.loadUrl(uri);
					}
				}
			}
		} catch (e) {
			BrimstoneErrorHandler.handleError("handleClick", e);		
		}
	},
	
	handleStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus) {
		try {
			var uri = aRequest.name;
		} catch(e) {
		}
		if (uri) {
			var match = /\w+:\/\/([^\/]+)(\/.+)/.exec(uri);
			if (match) {
				var host = match[1];
				var path = match[2];
				if (BrimstoneHandler.handlers[host]) {
			        if (aStateFlags & STATE_START) {
						if (BrimstoneHandler.handlers[host].handleStateStart) {
							BrimstoneHandler.handlers[host].handleStateStart(aWebProgress, aRequest)
						}
			        } else if (aStateFlags & STATE_REDIRECTING) {
						jsdump("I'm being redirected!");
						if (BrimstoneHandler.handlers[host].handleRedirect) {
							BrimstoneHandler.handlers[host].handleRedirect(aWebProgress, aRequest)
						}
					} else if (aStateFlags & STATE_STOP) {
						if (BrimstoneHandler.handlers[host].handleStateStop) {
							BrimstoneHandler.handlers[host].handleStateStop(aWebProgress, aRequest)
						}							
					}
				}
			}
		}
		return;		
	},
	
	beforeUnload : function(e) {
		var a = 1;
//		return false;
	},
	
	handlePage : function(host, path, DOM) {
		BrimstoneHandler.handlers[host].handlePage(host, path, DOM);
	},
	
}

// Removed BrimstoneListener for addProgressListener as now following 'load' event

function BrimstonePlayerUI(player, prefix) {
	this.player = player;
	this.name = document.getElementById(prefix + "Name");
	this.heroType = document.getElementById(prefix + "Type");
	this.faction = document.getElementById(prefix + "Faction");
	this.alliance = document.getElementById(prefix + "Alliance");
	this.size = document.getElementById(prefix + "Size");
	this.gold = document.getElementById(prefix + "Gold");
	this.attack = document.getElementById(prefix + "Attack");
	this.defence = document.getElementById(prefix + "Defence");
	this.maxDefence = document.getElementById(prefix + "MaxDefence");
	this.soldiers = document.getElementById(prefix + "Soldiers");
	this.knights = document.getElementById(prefix + "Knights");
	this.pikemen = document.getElementById(prefix + "Pikemen");
	this.elites = document.getElementById(prefix + "Elites");
	this.freeLand = document.getElementById(prefix + "FreeLand");
	this.mines = document.getElementById(prefix + "Mines");
	this.forts = document.getElementById(prefix + "Forts");
	this.trainingGrounds = document.getElementById(prefix + "TrainingGrounds");
	this.barracks = document.getElementById(prefix + "Barracks");
	this.amplifiers = document.getElementById(prefix + "Amplifiers");
	this.barriers = document.getElementById(prefix + "Barriers");	
}

BrimstonePlayerUI.prototype.clear = function() {
	this.name.value = ""
	this.heroType.value = "";
	this.faction.value = "";
	this.alliance.value = "";
	this.size.value = "";	
	this.gold.value = "";	
	this.soldiers.value = "";
	this.knights.value = "";
	this.pikemen.value = "";
	this.elites.value = "";
	this.attack.value = "";
	this.defence.value = "";
	if (this.maxDefence) {
		this.maxDefence.value = "";
	}
	this.freeLand.value = "";
	this.mines.value = "";
	this.forts.value = "";
	this.trainingGrounds.value = "";
	this.barracks.value = "";
	this.amplifiers.value = "";
	this.barriers.value = "";
}

BrimstonePlayerUI.prototype.update = function() {
	try {
		if (this.player) {
			if (this.name.value != this.player.name) {
				this.clear();
			}
			if (this.player.name) {
				this.name.value = this.player.name;
				this.heroType.value = "Level {0} {1}".format(this.player.level, this.player.heroType);
				this.faction.value = this.player.faction;
				if (this.player.allianceURL) {
					this.alliance.value = Brimstone.alliances[this.player.allianceURL].tag;
				}
				this.size.value = formatNumberAsString(0, this.player.size);
				if (this.player.gold == "TONS") {
					this.gold.value = "> 10 million"
				} else {
					this.gold.value = formatNumberAsString(0, this.player.gold);
				}
			}
			if (this.player.army) {
				var army = this.player.army;
				this.soldiers.value = formatNumberAsString(0 ,army.soldiers);
				this.knights.value = formatNumberAsString(0 ,army.knights);
				this.pikemen.value = formatNumberAsString(0 ,army.pikemen);
				this.elites.value = formatNumberAsString(0 ,army.elites);
				if (this.player.heroType) {
					var armyStats = this.player.modifiedArmyStrength;
					this.attack.value = formatNumberAsString(0 ,armyStats.attack);
					this.defence.value = formatNumberAsString(0 ,armyStats.defence);
					if (this.maxDefence) {
						var maxArmyStats = this.player.maxPossibleArmyStrength;
						this.maxDefence.value = formatNumberAsString(0 ,maxArmyStats.defence);
					}
				}
			}
			if (this.player.kingdom) {
				var kingdom = this.player.kingdom;
				this.freeLand.value = formatNumberAsString(0 ,kingdom.freeLand);
				this.mines.value = formatNumberAsString(0 ,kingdom.mines);
				this.forts.value = formatNumberAsString(0 ,kingdom.forts);
				this.trainingGrounds.value = formatNumberAsString(0, kingdom.trainingGrounds);
				this.barracks.value = formatNumberAsString(0, kingdom.barracks);
				this.amplifiers.value = formatNumberAsString(0, kingdom.amplifiers);
				this.barriers.value = formatNumberAsString(0, kingdom.barriers);
			}
		}
		if (BrimstoneOptions.hideBlankValues) {
			for (var memberName in this) {
				var member = this[memberName];
				if (member && member.localName == "label" && memberName !="name") {
					switch (member.value) {
						case null :
						case "" :
						case "0" :
							var collapsed = true;
							break;
						default :
							var collapsed = false;
					}
					member.parentNode.setAttribute("collapsed", collapsed);
				}
			}
		}
	} catch(e) {
		BrimstoneErrorHandler.handleError("updating player info", e);
	}
}

BrimstonePlayerUI.prototype.addDefenceStyle = function() {
	if (this.player && this.player.army) {
		this.defence.setAttribute("class", BrimstoneUI.getDefenceStyle(this.player.modifiedArmyStrength.defence));	
		this.maxDefence.setAttribute("class", BrimstoneUI.getDefenceStyle(this.player.maxPossibleArmyStrength.defence));	
	}
}

BrimstonePlayerUI.prototype.showValues = function(show) {
	for (var memberName in this) {
		var member = this[memberName];
		if (member && member.localName == "label" && memberName !="name") {
			member.parentNode.setAttribute("collapsed", !show);
		}
	}	
}

var BrimstoneBuildingCalculatorUI = {
	
	elementMap : {
		freeLand : {},
		mines : {},
		forts : {},
		trainingGrounds : {},
		barracks : {},
		amplifiers : {},
		barriers : {},
	},
	
	calculationMap : {
		income : {},
		maintenance :{}
	},
	
	targetNames : [],
	
	calculatorParameters : null,
	
	initialise : function() {
		for (var name in BrimstoneBuildingCalculatorUI.elementMap) {
			var map = BrimstoneBuildingCalculatorUI.elementMap[name];
			map.current = document.getElementById("current" + name.upperCaseFirst());
			var targetName = "target" + name.upperCaseFirst();
			map.target = document.getElementById(targetName);
			BrimstoneBuildingCalculatorUI[targetName] = map.target;
			BrimstoneBuildingCalculatorUI.targetNames.push(targetName);
			map.delta = document.getElementById(name + "Delta");
		}
		for (var name in BrimstoneBuildingCalculatorUI.calculationMap) {
			var map = BrimstoneBuildingCalculatorUI.calculationMap[name];
			map.current = document.getElementById("current" + name.upperCaseFirst());
			var targetName = "target" + name.upperCaseFirst();
			map.target = document.getElementById(targetName);
		}
		BrimstoneBuildingCalculatorUI.parameters = BrimstoneData.loadParameters(BrimstoneBuildingCalculatorUI.targetNames, true);
	},
	
	factionChanged : function() {
		BrimstoneBuildingCalculatorUI.factionParameters = BrimstoneBuildingCalculatorUI.parameters[Brimstone.faction];
		for (var parameterName in BrimstoneBuildingCalculatorUI.factionParameters) {
			if (BrimstoneBuildingCalculatorUI.factionParameters[parameterName].value) {
				BrimstoneBuildingCalculatorUI[parameterName].value = BrimstoneBuildingCalculatorUI.factionParameters[parameterName].value;
			}
		}		
		BrimstoneBuildingCalculatorUI.targetChanged();
	},
	
	calculateFields : function(size, currentCount, currentElem, targetElem, deltaElem) {
		var currentPercent = (100 * currentCount) / size;
		currentElem.value = Math.roundTo(currentPercent, 1);
		var targetPercent = targetElem.value;
		if (targetPercent) {
			var targetCount = (size * targetPercent) / 100;
			var delta = targetCount - currentCount;
			deltaElem.value = Math.ceil(delta);
		}
	},
	
	targetChanged : function(e) {
		var targetTotal = 0;
		for (var name in BrimstoneBuildingCalculatorUI.elementMap) {
			var map = BrimstoneBuildingCalculatorUI.elementMap[name];
			if (name != "freeLand") {
				if (map.target.value) {
					var targetValue = parseFloat(map.target.value);
				} else {
					var targetValue = 0;
				}
				map.target.value = targetValue
				targetTotal += targetValue;
			}
		}
		BrimstoneBuildingCalculatorUI.elementMap.freeLand.target.value = Math.roundTo(100 - targetTotal, 1);
		BrimstoneBuildingCalculatorUI.update(Brimstone.player);
		if (e && BrimstoneBuildingCalculatorUI.factionParameters) {
			var targetName = e.target.id
			if (!BrimstoneBuildingCalculatorUI.factionParameters[targetName]) {
				BrimstoneBuildingCalculatorUI.factionParameters[targetName] = new BrimstoneParameter(targetName);
			}
			var parameter = BrimstoneBuildingCalculatorUI.factionParameters[targetName];
			parameter.value = BrimstoneBuildingCalculatorUI[targetName].value;
			parameter.save();
		}
	},
	
	update : function(player) {
		if (!(player && player.size && player.kingdom)) {
			return;
		}
		var size = player.size;
		var kingdom = player.kingdom;

		var dummyPlayer = new playerInfo();
		dummyPlayer.size = player.size;
		var dummyKingdom = new kingdomInfo();
		dummyPlayer.kingdom = dummyKingdom;
		
		for (var name in BrimstoneBuildingCalculatorUI.elementMap) {
			var map = BrimstoneBuildingCalculatorUI.elementMap[name];
			dummyKingdom[name] = map.target.value * size / 100;
			BrimstoneBuildingCalculatorUI.calculateFields(size, kingdom[name], map.current, map.target, map.delta);
		}
		
		dummyKingdom.calculateRatios();
		
		BrimstoneBuildingCalculatorUI.calculationMap.income.current.value = formatNumberAsString(0, Math.ceil(player.income));
		BrimstoneBuildingCalculatorUI.calculationMap.income.target.value = formatNumberAsString(0, Math.ceil(dummyPlayer.income));
		BrimstoneBuildingCalculatorUI.calculationMap.maintenance.current.value = formatNumberAsString(0, Math.ceil(player.armyMaintenance));
	}
}

var BrimstoneUI =  {
	sidebar : null,
	log : null,
	tabPanels : null,
	tabBox : null,
	playersListTree : null,
	playersListTreeView : null,
	playersListChildren : null,
	playersListDefenceColumn : null,
	playersListMaxDefenceColumn : null,
	eventsTree : null,
	eventsTreeView : null,
	playerUI : null,
	targetUI : null,
	playerFilterAge : 1,

	initialise : function() {
		try {
			BrimstoneUI.sidebar = Brimstone.parentWindow.document.getElementById("sidebar");
			if (BrimstoneOptions.noMaxWidth) {
				BrimstoneUI.setSidebarMaxWidth("none");
			};
			BrimstoneUI.log = document.getElementById("log");
			BrimstoneUI.playerUI = new BrimstonePlayerUI(Brimstone.player, "player");
			BrimstoneUI.targetUI = new BrimstonePlayerUI(null, "targets");
			BrimstoneUI.targetUI.showValues(!BrimstoneOptions.hideBlankValues);
			BrimstoneUI.tabPanels = document.getElementById("tabPanels");
			BrimstoneUI.tabBox = document.getElementById("tabBox");
			BrimstoneUI.playersList = document.getElementById("playersList");
			BrimstoneUI.playersListTree = document.getElementById("playersListTree");
			var DBConn = BrimstoneData.xuluData.newConnection();
			BrimstoneData.xuluData.executeStatement("CREATE TEMP VIEW testView AS SELECT name, heroType, level, size, gold, faction, tag, playerCalculations.attack, playerCalculations.defence, playerCalculations.maxAttack, playerCalculations.maxDefence, timestamp FROM players LEFT JOIN alliances ON players.allianceURL = alliances.url LEFT JOIN playerCalculations ON name=playerName", null, DBConn);
			var ds = BrimstoneData.xuluData.addDataStore(playerDisplay);
			ds.addEventListener = function(event, listener) {
				playerInfo.dataStore.addEventListener(event, listener);
			}

			ds.removeEventListener = function(event, listener) {
				playerInfo.dataStore.removeEventListener(event, listener);
			}
			BrimstoneUI.playersListTreeView = new XuluTreeView(BrimstoneUI.playersListTree, null, playerDisplayTreeMap, DBConn);
			BrimstoneUI.playersListTreeView.groupByColumnIndex = 1;
			BrimstoneUI.playersListTree.view = BrimstoneUI.playersListTreeView;
			BrimstoneUI.playersListDefenceColumn = BrimstoneUI.playersListTreeView.treebox.columns.getNamedColumn("defence");
			BrimstoneUI.playersListMaxDefenceColumn = BrimstoneUI.playersListTreeView.treebox.columns.getNamedColumn("maxDefence");

			var DBConn = BrimstoneData.xuluData.newConnection();
			BrimstoneData.xuluData.executeStatement("CREATE TEMP VIEW eventsView AS SELECT id, displayName, eventDescription, acres, gold, message, subject, object, timestamp FROM events LEFT JOIN eventDisplayFields ON id = eventDisplayFields.eventId LEFT JOIN eventDeltas ON id = eventDeltas.eventId AND displayDelta = actor", null, DBConn);
			var ds = BrimstoneData.xuluData.addDataStore(eventsDisplay);
			ds.addEventListener = function(event, listener) {
				brimstoneEvent.dataStore.addEventListener(event, listener);
			}

			ds.removeEventListener = function(event, listener) {
				brimstoneEvent.dataStore.removeEventListener(event, listener);
			}
			BrimstoneUI.eventsTree = document.getElementById("eventsTree");
			BrimstoneUI.eventsTreeView = new XuluTreeView(BrimstoneUI.eventsTree, null, eventsDisplayTreeMap, DBConn);
			BrimstoneUI.eventsTree.view = BrimstoneUI.eventsTreeView;
			
			BrimstoneBuildingCalculatorUI.initialise();
		} catch(e) {
			BrimstoneErrorHandler.handleError("initialising ui", e);			
		}		
	},
	
	setSidebarMaxWidth : function(width) {
		if (BrimstoneUI.sidebar) {
			/* This works in 3.0 but not 3.5
//			BrimstoneUI.origSidebarMaxWidth = BrimstoneUI.sidebar.style["maxWidth"];
//			BrimstoneUI.sidebar.style["maxWidth"] = width;
			Use the following instead*/
			BrimstoneUI.origSidebarMaxWidth = BrimstoneUI.sidebar.style.getPropertyValue("max-width");
			BrimstoneUI.sidebar.style.setProperty("max-width", width, "important");
		}
	},

	resetSidebarMaxWidth : function() {
		if (BrimstoneUI.origSidebarMaxWidth) {
			/* This works in 3.0 but not 3.5
//			BrimstoneUI.sidebar.style["maxWidth"] = BrimstoneUI.origSidebarMaxWidth;
			Use the following instead*/
			BrimstoneUI.sidebar.style.setProperty("max-width", BrimstoneUI.origSidebarMaxWidth, "important");
			delete BrimstoneUI.origSidebarMaxWidth;
		}
	},
	
	terminate : function() {
		BrimstoneUI.resetSidebarMaxWidth();
		BrimstoneUI.playersListTreeView.terminate();
		BrimstoneUI.playersListTreeView = null;
		BrimstoneUI.eventsTreeView.terminate();
		BrimstoneUI.eventsTreeView = null;
	},
	
	clear : function() {
		BrimstoneUI.playerUI.clear();
		BrimstoneUI.targetUI.clear();
		BrimstoneUI.eventsTreeView.dataSource = null;
		BrimstoneUI.playersListTreeView.dataSource = null;
	},
	
	
	appendLog : function(text) {
//		BrimstoneUI.log.value += text;
	},
	
	updatePlayerInfo : function() {
		try {
			BrimstoneUI.playerUI.player = Brimstone.player;
			BrimstoneUI.playerUI.update();
		} catch(e) {
			BrimstoneErrorHandler.handleError("updating player info", e);
		}
	},
	
	playerFilterChanged : function() {
		var playerFilter = document.getElementById("playersFilterSelection");
		var selectedItem = playerFilter.selectedItem;
		BrimstoneUI.playerFilterAge = selectedItem.value;
		if (BrimstoneUI.playerFilterAge) {
			BrimstoneUI.playersListTreeView.filter = {
				returnType : "indexedByPrimaryKey",
				whereClause : "WHERE name <> ?1 AND timestamp > ?2",
				whereValues : [
					{type : "string", value : Brimstone.player.name},
					{type : "string", value : getTimeStamp(-BrimstoneUI.playerFilterAge)}
				]
			}			
		} else {
			BrimstoneUI.playersListTreeView.filter = {
				returnType : "indexedByPrimaryKey",
				whereClause : "WHERE name <> ?1",
				whereValues : [
					{type : "string", value : Brimstone.player.name},
				]
			}
			
		}
	},

	eventFilterChanged : function() {
		var playerName = Brimstone.player.name;
		var eventFilter = document.getElementById("eventsFilterSelection");
		var selectedItem = eventFilter.selectedItem;
		BrimstoneUI.eventFilterAge = selectedItem.value;
		if (BrimstoneUI.eventFilterAge) {
			BrimstoneUI.eventsTreeView.filter = {
				whereClause: "WHERE (subject = ?1 OR object = ?2) AND timestamp > ?3",
				whereValues: [
					{type : "string", value : playerName}, 
					{type : "string", value : playerName},
					{type : "string", value : getTimeStamp(-BrimstoneUI.eventFilterAge)}
				]
			} 
		} else {
			BrimstoneUI.eventsTreeView.filter = {
				whereClause: "WHERE subject = ?1 OR object = ?2",
				whereValues: [{
					type: "string",
					value: playerName
				}, {
					type: "string",
					value: playerName
				}]
			} 			
		}
	},
		
	playerChanged : function() {
		var playerName = Brimstone.player.name

		BrimstoneUI.eventFilterChanged();

		BrimstoneUI.eventsTreeView.dataSource = eventsDisplay.dataStore;

		BrimstoneUI.playerFilterChanged();

		BrimstoneUI.playersListTreeView.dataSource = playerDisplay.dataStore;
		
		BrimstoneBuildingCalculatorUI.factionChanged();

		BrimstoneUI.updatePlayerInfo();
	},

	updateTargetInfo : function(player) {
		try {
			BrimstoneUI.targetUI.player = player;
			BrimstoneUI.targetUI.update();
			BrimstoneUI.targetUI.addDefenceStyle();
	//		BrimstoneUI.tabPanels.selectedIndex = 1;
			BrimstoneUI.tabBox.selectedIndex = 1;
		} catch(e) {
			BrimstoneErrorHandler.handleError("updating target info", e)
		}
	},
	
	playerListDblClick : function(e) {
		try {
			var tbo = BrimstoneUI.playersListTree.treeBoxObject;
			var rowIndex = tbo.getRowAt(e.clientX, e.clientY);
			if (rowIndex >= 0 ) {
				var rowObject = BrimstoneUI.playersListTreeView.getRowObject(rowIndex);
				var playerName = rowObject.name;
				var player = BrimstoneData.loadPlayer(playerName);
				BrimstoneUI.updateTargetInfo(player);
				if (player.id  && BrimstoneOptions.loadPageOnDblClick) {
					BrimstoneBrowser.loadPage("http://{0}/diplomacy/userPage?target={1}&secure=1".format(BrimstoneOptions.warbookHost, player.id));
				}
			}
		} catch(e) {
			BrimstoneErrorHandler.handleError("showing target", e)
		}
	},

	eventsDblClick : function(e) {
		try {
			var tbo = BrimstoneUI.eventsTree.treeBoxObject;
			var rowIndex = tbo.getRowAt(e.clientX, e.clientY);
			if (rowIndex >= 0 ) {
				var rowObject = BrimstoneUI.eventsTreeView.getRowObject(rowIndex);
				var playerName  = rowObject.displayName;
//				var row = BrimstoneUI.playersListChildren.childNodes[rowIndex].firstChild;
//				var playerName = row.firstChild.getAttribute("label");
				if (playerName) {
					
					var player = BrimstoneData.loadPlayer(playerName);
					if (player) {
						BrimstoneUI.updateTargetInfo(player);
						if (player.id && BrimstoneOptions.loadPageOnDblClick) {
							BrimstoneBrowser.loadPage("http://{0}/diplomacy/userPage?target={1}&secure=1".format(BrimstoneOptions.warbookHost, player.id));
						}
					}
				}
			}
		} catch(e) {
			BrimstoneErrorHandler.handleError("showing target", e)
		}
	},
		
	saveTarget : function() {
		try {
			BrimstoneData.savePlayer(BrimstoneUI.targetUI.player);
		} catch(e) {
			BrimstoneErrorHandler.handleError("saving target", e)
		}
	},
	
	showOptions : function() {
		var optionsWindow = window.open("chrome://brimstone/content/options.xul",
		    "Options",
    		"chrome, dialog, modal, resizable=no,scrollbars=no,status=no");
	},
	
	keyPressed : function(e) {
//		jsdump("id: {2} ctrl: {0} key: {1}".format((e.ctrlKey) ? true : false, e.which, e.target.id));
		if (e.ctrlKey && e.which == 99) {
			BrimstoneUI.copyUI(e.target.id)
		}
	},
	
	contextMenuShowing : function() {
//var a =		document.popupNode;
	},
	
	copyClicked : function(e) {
		var copyTarget = document.popupNode;
		// We can only copy focusable elements
		while (!copyTarget.style.MozUserFocus && copyTarget.parentNode) {
			copyTarget = copyTarget.parentNode;
		}
		
		BrimstoneUI.copyUI(copyTarget.id);
	},
	
	copyUI : function(elementID) {
		switch (elementID) {
			case "playerContainer" : 
				BrimstoneUI.copyPlayerUItoClipboard(BrimstoneUI.playerUI);
				break;
			case "targetContainer" : 
				BrimstoneUI.copyPlayerUItoClipboard(BrimstoneUI.targetUI);
				break;
		}
		
	},
	
	copyPlayerUItoClipboard : function(playerUI) {
		var copyString = "";
		for (var memberName in playerUI) {
			var member = playerUI[memberName]
			if (member && member.localName == "label") {
				copyString += "{0}\t{1}\n".format(memberName, member.value)
			}
		}
		gClipboardHelper.copyString(copyString); 		
	},
	
	testClicked : function() {
		var tree = document.getElementById("my-tree");
		var DBConn = BrimstoneData.xuluData.newConnection();
		BrimstoneData.xuluData.executeStatement("CREATE TEMP VIEW testView AS SELECT name, heroType, level, size, gold, faction, tag, playerCalculations.attack, playerCalculations.defence FROM players LEFT JOIN alliances ON players.allianceURL = alliances.url LEFT JOIN playerCalculations ON name=playerName", null, DBConn);
//		BrimstoneData.xuluData.executeStatement("CREATE TEMP TABLE testTable AS SELECT name, url, tag FROM players LEFT JOIN alliances ON players.allianceURL = alliances.url", null, DBConn);
		var ds = BrimstoneData.xuluData.addDataStore(testPlayer);
//		ds.dataMap._selectClauseTemplate = "SELECT players.name, p//layers.allianceURL, alliances.tag FROM players, alliances WHERE players.allianceURL = alliances.url {0} {1}"
		var a = 1;
		var treeView = new XuluTreeView(tree, ds, testPlayersTreeMap, DBConn);
//		var treeView = new XuluTreeView(tree, brimstoneEvent.dataStore, eventsTreeMap, null);
		var a = 1;
		tree.view = treeView
//		treeView.objects = Brimstone.events;
/*		treeView.filter = {
			whereClause : "WHERE subject = ?1 OR object = ?2",
			whereValues : [
				{type : "string", value : Brimstone.player.name},
				{type : "string", value : Brimstone.player.name}
			]
		} */
//		document.getElementById('my-tree').view=treeView;

//		for (var i=0, l=BrimstoneUI.playerUI.childNodes.length; i < l; i++) {
//			jsdump(BrimstoneUI.playerUI.childNodes[i].firstChild.value);
//		}
	},
	
	getDefenceStyle : function(defence) {
		if (Brimstone.player.modifiedArmyStrength.attack > defence * (1 + (BrimstoneOptions.easyTargetTolerance / 100))) {
			return "makeItGreen";		
		} else if (Brimstone.player.modifiedArmyStrength.attack < defence * (1 + (BrimstoneOptions.improbableTargetTolerance / 100))) {
			return "makeItRed";		
		} else {
			return "makeItYellow";				
		}
		
	},
	
	getDefenceStyleProperties : function(col, row, value) {
		var defence = parseInt(value);
		if (Brimstone.player && defence != 0) {
			return [BrimstoneUI.getDefenceStyle(defence)];
		} else {
			return [];
		}
	},
	
	invalidateDefenceStyling : function() {
		BrimstoneUI.playersListTreeView.treebox.invalidateColumn(BrimstoneUI.playersListDefenceColumn);
		BrimstoneUI.playersListTreeView.treebox.invalidateColumn(BrimstoneUI.playersListMaxDefenceColumn);		
		BrimstoneUI.targetUI.addDefenceStyle();
	}
}

function playerDisplay() {
	
}
playerDisplay.dataMap = {
	tableName : "testView",
	primaryKey : ["name"],
	primaryKeyLookup : {"name" : true},
	fieldMappings : {
		"name" : {member: "name", type : "string", fieldName : "name"},
		"heroType" : {member: "heroType", type : "string", fieldName : "heroType"},
		"level" : {member: "level", type : "integer", fieldName : "level"},
		"size" : {member: "size", type : "integer", fieldName : "size"},
		"gold" : {member: "gold", type : "integer", fieldName : "gold"},
		"faction" : {member: "faction", type : "string", fieldName : "faction"},
		"allianceTag" : {member: "allianceTag", type : "string", fieldName : "tag"},
		"attack" : {member: "attack", type : "integer", fieldName : '"playerCalculations.attack"'},
		"defence" : {member: "defence", type : "integer", fieldName : '"playerCalculations.defence"'},
		"maxAttack" : {member: "maxAttack", type : "integer", fieldName : '"playerCalculations.maxAttack"'},
		"maxDefence" : {member: "maxDefence", type : "integer", fieldName : '"playerCalculations.maxDefence"'},
	}
}

var playerDisplayTreeMap = {
	columns : [
		{ columnName : "name", label : "Name", type : "string", member : "name"},
		{ columnName : "heroType", label : "Hero", type : "string", member : "heroType"},
		{ columnName : "level", label : "Level", type : "integer", member : "level"},
		{ columnName : "size", label : "Size", type : "integer", member : "size"},
		{ columnName : "gold", label : "Gold", type : "integer", member : "gold"},
		{ columnName : "faction", label : "Faction", type : "string", member : "faction"},
		{ columnName : "allianceTag", label : "Alliance", type : "string", member : "allianceTag"},
		{ columnName : "attack", label : "Attack", type : "integer", member : "attack"},
		{ columnName : "defence", label : "Defence", type : "integer", member : "defence", getStyleProperties: BrimstoneUI.getDefenceStyleProperties},
		{ columnName : "maxAttack", label : "Max Attack", type : "integer", member : "maxAttack"},
		{ columnName : "maxDefence", label : "Max Defence", type : "integer", member : "maxDefence", getStyleProperties: BrimstoneUI.getDefenceStyleProperties},
	]
}

function eventsDisplay() {
	
}

eventsDisplay.dataMap = {
	tableName : "eventsView",
	primaryKey : ["id"],
	primaryKeyLookup : {"id" : true},
	fieldMappings : {
		"id" : {member: "id", type : "integer", fieldName : "id"},
		"displayName" : {member: "displayName", type : "string", fieldName : "displayName"},
		"eventDescription" : {member: "eventDescription", type : "string", fieldName : "eventDescription"},
		"acres" : {member: "acres", type : "integer", fieldName : "acres"},
		"gold" : {member: "gold", type : "integer", fieldName : "gold"},
		"message" : {member: "message", type : "string", fieldName : "message"},
	}
}

var eventsDisplayTreeMap = {
	columns : [
		{ columnName : "id", label : "Id", type : "integer", member : "id"},
		{ columnName : "displayName", label : "Player", type : "string", member : "displayName"},
		{ columnName : "eventDescription", label : "Event", type : "string", member : "eventDescription"},
		{ columnName : "acres", label : "Acres", type : "integer", member : "acres"},
		{ columnName : "gold", label : "Gold", type : "integer", member : "gold"},
		{ columnName : "message", label : "Message", type : "string", member : "message"}
	]
}
var heroTypes = {
	Automator : {startAttack : 3, startDefence : 3},
	General : {startAttack : 3, startDefence : 3},
	Magician : {startAttack : 1, startDefence : 0},
	Mogul : {startAttack : 1, startDefence : 2},
	Necromancer : {startAttack : 2, startDefence : 2},
	Slayer : {startAttack : 4, startDefence : 1},
	Visionary : {startAttack : 0, startDefence : 5},
	Warmonger : {startAttack : 5, startDefence : 0}
}

var heroElites = {
	Automator : new unitStat("Machina", "Machina", 5, 5, 1000, 0.2),
	General : new unitStat("Elite", "Elites", 5, 5, 700, 3),
	Magician : new unitStat("Wizard", "Wizards", 5, 3, 400, 3),
	Mogul : new unitStat("Legionnaire", "Legionnaires", 3, 5, 400, 3),
	Necromancer : new unitStat("Zombie", "Zombies", 3, 3, 400, 2),
	Slayer : new unitStat("Assassin", "Assassins", 6, 2, 400, 3),
	Visionary : new unitStat("Defender", "Defenders", 2, 8, 500, 3),
	Warmonger : new unitStat("Berserker", "Berserker", 8, 2, 500, 3)
}

var unitStrengths = {
	soldier : new unitStat("soldier", "soldiers", 1, 1, 100, 1),
	knight : new unitStat("knight", "knights", 3, 1, 200, 2),
	pikeman : new unitStat("pikeman", "pikemen", 1, 3, 200, 2)
}

function playerCalculations(player) {
	this.player = player;
}

playerCalculations.dataMap = {
	tableName : "playerCalculations",
	primaryKey : ["playerName"],
	primaryKeyLookup : {"playerName" : true},
	fieldMappings : {
		"playerName" : {member: "playerName", type : "string", fieldName : "playerName"},
		"attack" : {member: "attack", type : "integer", fieldName : "attack"},
		"defence" : {member: "defence", type : "integer", fieldName : "defence"},
		"maxAttack" : {member: "maxAttack", type : "integer", fieldName : "maxAttack"},
		"maxDefence" : {member: "maxDefence", type : "integer", fieldName : "maxDefence"},
	}
}

playerCalculations.prototype.__defineGetter__("attack", function() {
	return this.player.modifiedArmyStrength.attack;	
});

playerCalculations.prototype.__defineGetter__("defence", function() {
	return this.player.modifiedArmyStrength.defence;	
});

playerCalculations.prototype.__defineGetter__("maxAttack", function() {
	return this.player.maxPossibleArmyStrength.attack;	
});

playerCalculations.prototype.__defineGetter__("maxDefence", function() {
	return this.player.maxPossibleArmyStrength.defence;	
});

function playerInfo () {
	this.name = null;
	this.id = null;
	this.level = null;
	this.heroType = null;
	this._size = null;
	this.gold = null;
	this.mana = null;
	this.faction = null;
	this.allianceURL = null;
	this.army = null;
	this._kingdom = null;
	this.stats = null;
	this.timestamp = null;
	this.calculations = new playerCalculations(this);
}

playerInfo.dataMap = {
	tableName : "players",
	primaryKey : ["name"],
	primaryKeyLookup : {"name" : true},
	fieldMappings : {
		"name" : {member: "name", type : "string", fieldName : "name"},
		"id" : {member: "id", type : "string", fieldName : "id"},
		"level" : {member: "level", type : "integer", fieldName : "level"},
		"heroType" : {member: "heroType", type : "string", fieldName : "heroType"},
		"size" : {member: "size", type : "integer", fieldName : "size"},
		"gold" : {member: "gold", type : "integer", fieldName : "gold"},
		"faction" : {member: "faction", type : "string", fieldName : "faction"},
		"allianceURL" : {member: "allianceURL", type : "string", fieldName : "allianceURL"},
		"timestamp" : {member: "timestamp", type : "string", fieldName : "timestamp"},
		"army" : {member: "army", type : "flattenedObject", createIfDefined : "soldiers", objectType : armyInfo},
		"kingdom" : {member: "kingdom", type : "flattenedObject", createIfDefined : "freeLand", objectType : kingdomInfo},
		"stats" : {member: "stats", type : "flattenedObject", createIfDefined : "attack", objectType : Stats},
		"calculations" : {member: "calculations", type : "calculatedObject", objectType : playerCalculations, pkMap : {"playerName" : "name"}}		
	}
}

// 		"objectDelta" : {member: "objectDelta", type : "dependentObject", objectType : eventDelta, pkMap : {"eventId" : "id", "actor" : "object"}}
playerInfo.prototype.__defineGetter__("eliteName", function() {
	return heroElites[this.heroType];
});

playerInfo.prototype.__defineGetter__("eliteInfo", function() {
	return heroElites[this.heroType];
});

playerInfo.prototype.__defineGetter__("allianceTag", function() {
	if (this.allianceURL) {
		return Brimstone.alliances[this.allianceURL].tag;
	}
});

function addArmyStats(total, stat, multiple) {
	total.attack += stat.attack * multiple;
	total.defence += stat.defence * multiple;	
}

playerInfo.prototype.__defineGetter__("baseArmyStrength", function() {
	var armyStats = new unitStat("army", "", 0, 0);
	if (this.army) {
		addArmyStats(armyStats, unitStrengths.soldier, this.army.soldiers);
		addArmyStats(armyStats, unitStrengths.knight, this.army.knights);
		addArmyStats(armyStats, unitStrengths.pikeman, this.army.pikemen);
		addArmyStats(armyStats, this.eliteInfo, this.army.elites);
		if (this.size) {
			armyStats.defence += this.size;
		}
	}
	return armyStats;
});

playerInfo.prototype.__defineGetter__("modifiedArmyStrength", function() {
	var armyStats = this.baseArmyStrength;
	var modifier = new unitStat("modifier", "", 1, 1);
	if (this.kingdom) {
		var fortsRatio = this.kingdom.fortsRatio;
		if (fortsRatio > 0.3) {
			fortsRatio = 0.3;
		}
		modifier.defence += fortsRatio * 1.5;
		var trainingGroundsRatio = this.kingdom.trainingGroundsRatio / this.size;
		if (trainingGroundsRatio > 0.3) {
			trainingGroundsRatio = 0.3;
		}
		modifier.attack += trainingGroundsRatio * 1.5;
	}
	if (this.stats) {
		modifier.attack += this.stats.attack / 100;
		modifier.defence += this.stats.defence / 100;
	}
	armyStats.attack *= modifier.attack;
	armyStats.attack = Math.ceil(armyStats.attack);
	armyStats.defence *= modifier.defence;
	armyStats.defence = Math.ceil(armyStats.defence);
	
	return armyStats;
});

playerInfo.prototype.__defineGetter__("maxPossibleArmyStrength", function() {
	var armyStats = this.baseArmyStrength;
	var modifier = new unitStat("modifier", "", 1, 1);
	if (this.kingdom) {
		var fortsRatio = this.kingdom.fortsRatio;
		if (fortsRatio > 0.3) {
			fortsRatio = 0.3;
		}
		modifier.defence += fortsRatio * 1.5;
		var trainingGroundsRatio = this.kingdom.trainingGroundsRatio / this.size;
		if (trainingGroundsRatio > 0.3) {
			trainingGroundsRatio = 0.3;
		}
		modifier.attack += trainingGroundsRatio * 1.5;
	} else {
		modifier.defence += 0.45;
		modifier.attack += 0.45;
	}	
	var maxAttackStat = heroTypes[this.heroType].startAttack + (this.level - 1) * 2;
	var maxDefenceStat = heroTypes[this.heroType].startDefence + (this.level - 1) * 2;
	modifier.attack += maxAttackStat / 100;
	modifier.defence += maxDefenceStat / 100;

	armyStats.attack *= modifier.attack;
	armyStats.attack = Math.ceil(armyStats.attack);
	armyStats.defence *= modifier.defence;
	armyStats.defence = Math.ceil(armyStats.defence);
	
	return armyStats
});

playerInfo.prototype.__defineGetter__("armyMaintenance", function() {
	var maintenance = 0;
	if (this.name && this.army) {
		maintenance += 	this.army.soldiers * unitStrengths.soldier.maintenance;
		maintenance += 	this.army.pikemen * unitStrengths.pikeman.maintenance;
		maintenance += 	this.army.knights * unitStrengths.knight.maintenance;
		maintenance += 	this.army.elites * this.eliteInfo.maintenance;
	}
	return maintenance;
});

playerInfo.prototype.__defineGetter__("income", function() {
	var income = 0;
	var tradeBenefits = 0.15; // This appears to be broken in Warbook and you always get trade benefits event if you have no trade agreements
	if (this.kingdom) {
		income = 	(this.kingdom.mines * 30 + this.size * 10) * (1 + tradeBenefits);
	}
	return income;
});

playerInfo.prototype.__defineGetter__("size", function() {
	return this._size;
});

playerInfo.prototype.__defineSetter__("size", function(size) {
	if (this._size != size) {
		this._size = size;
		if (this.kingdom) {
			this.kingdom.size = size;
		}
	}
});

playerInfo.prototype.__defineGetter__("kingdom", function() {
	return this._kingdom;
});

playerInfo.prototype.__defineSetter__("kingdom", function(kingdom) {
	this._kingdom = kingdom;
	kingdom.size = this.size;
});
function Stats() {
	this.attack = null;	
	this.defence = null;
	this.spellPower = null;
	this.maxMana = null;
}

Stats.dataMap = {
	tableName : "players",
	primaryKey : [],
	primaryKeyLookup : {},
	fieldMappings : {
		"attack" : {member: "attack", type : "integer", fieldName : "attack"},
		"defence" : {member: "defence", type : "integer", fieldName : "defence"},
		"spellPower" : {member: "spellPower", type : "integer", fieldName : "spellPower"},
		"maxMana" : {member: "maxMana", type : "integer", fieldName : "maxMana"},
	}
}

var statsParseHints = [
	{re:/Attack: ([\d,]+)/, attributeName:"attack", attributeType: "number"},
	{re:/Defense: ([\d,]+)/, attributeName:"defence", attributeType: "number"},
	{re:/Spell Power: ([\d,]+)/, attributeName:"spellPower", attributeType: "number"},
	{re:/Mana: [\d,]+ \/ ([\d,]+)/, attributeName:"maxMana", attributeType: "number"},
];

function armyInfo () {
	this.soldiers = null;
	this.pikemen = null;
	this.knights = null;
	this.elites = null;
}

armyInfo.dataMap = {
	tableName : "players",
	primaryKey : [],
	primaryKeyLookup : {},
	fieldMappings : {
		"soldiers" : {member: "soldiers", type : "integer", fieldName : "soldiers"},
		"pikemen" : {member: "pikemen", type : "integer", fieldName : "pikemen"},
		"knights" : {member: "knights", type : "integer", fieldName : "knights"},
		"elites" : {member: "elites", type : "integer", fieldName : "elites"},
	}
}

var armyInfoParseHints = [
	{re:/([\d,]+) Soldiers/, attributeName:"soldiers", attributeType: "number"},
	{re:/([\d,]+) Knights/, attributeName:"knights", attributeType: "number"},
	{re:/([\d,]+) Pikemen/, attributeName:"pikemen", attributeType: "number"},
	{re:/([\d,]+) Elites/, attributeName:"elites", attributeType: "number"}
];

function unitStat(name, plural, attack, defence, cost, maintenance) {
	this.name = name;
	this.plural = plural;
	this.attack = attack;
	this.defence = defence;
	this.cost = cost;
	this.maintenance = maintenance;
}

function kingdomInfo() {
	this._size = null;
	this.freeLand = null;
	this.mines = null;
	this.forts = null;
	this.trainingGrounds = null;
	this.barracks = null;
	this.amplifiers = null;
	this.barriers = null;
}

kingdomInfo.dataMap = {
	tableName : "players",
	primaryKey : [],
	primaryKeyLookup : {},
	fieldMappings : {
		"freeLand" : {member: "freeLand", type : "integer", fieldName : "freeLand"},
		"mines" : {member: "mines", type : "integer", fieldName : "mines"},
		"forts" : {member: "forts", type : "integer", fieldName : "forts"},
		"trainingGrounds" : {member: "trainingGrounds", type : "integer", fieldName : "trainingGrounds"},
		"barracks" : {member: "barracks", type : "integer", fieldName : "barracks"},
		"amplifiers" : {member: "amplifiers", type : "integer", fieldName : "amplifiers"},
		"barriers" : {member: "barriers", type : "integer", fieldName : "barriers"},
	}
}

kingdomInfo.prototype.calculateRatios = function(){
	this.freeLandRatio = this.freeLand / this.size;
	this.minesRatio = this.mines / this.size;
	this.fortsRatio = this.forts / this.size;
	this.trainingGroundsRatio = this.trainingGrounds / this.size;
	this.barracksRatio = this.barracks / this.size;
	this.amplifiersRatio = this.amplifiers / this.size;
	this.barriersRatio = this.barriers / this.size;	
}

kingdomInfo.prototype.__defineGetter__("size", function() {
	return this._size;
});

kingdomInfo.prototype.__defineSetter__("size", function(size) {
	if (this._size != size) {
		this._size = size;
		this.calculateRatios();
	}
});

var kingdomInfoParseHints = [
	{re:/([\d,]+) acres of free land/, attributeName:"freeLand", attributeType: "number"},
	{re:/([\d,]+) mines/, attributeName:"mines", attributeType: "number"},
	{re:/([\d,]+) forts/, attributeName:"forts", attributeType: "number"},
	{re:/([\d,]+) training grounds/, attributeName:"trainingGrounds", attributeType: "number"},
	{re:/([\d,]+) barracks/, attributeName:"barracks", attributeType: "number"},
	{re:/([\d,]+) amplifiers/, attributeName:"amplifiers", attributeType: "number"},
	{re:/([\d,]+) barriers/, attributeName:"barriers", attributeType: "number"}
];

function eventDelta() {
	this.acres = null;
	this.gold = null;
	this.troops = null;
	this.soldiers = null;
	this.pikemen = null;
	this.knights = null;
	this.elites = null;
}

eventDelta.dataMap = {
	tableName : "eventDeltas",
	primaryKey : ["eventId", "actor"],
	primaryKeyLookup : {"eventId" : true, "actor" : true},
	fieldMappings : {
		"eventId" : {member: "eventId", type : "integer", fieldName : "eventId"},
		"actor" : {member: "actor", type : "string", fieldName : "actor"},
		"acres" : {member: "acres", type : "string", fieldName : "acres"},
		"gold" : {member: "gold", type : "string", fieldName : "gold"},
		"troops" : {member: "troops", type : "string", fieldName : "troops"},
		"soldiers" : {member: "soldiers", type : "string", fieldName : "soldiers"},
		"pikemen" : {member: "pikemen", type : "string", fieldName : "pikemen"},
		"knights" : {member: "knights", type : "string", fieldName : "knights"},
		"elites" : {member: "elites", type : "string", fieldName : "elites"}
	}
}

brimstoneEvent.prototype.__defineGetter__("displayFields", function() {
	if (!this._displayFields) {
		this._displayFields = new brimstoneEventDisplayFields(this);
	}
	return this._displayFields;
});

brimstoneEventDisplayFields.prototype.calculateDisplayFields = function(theEvent){
	var playerName = theEvent.subject;
	if (playerName == Brimstone.player.name) {
		var userRole = "active";
		playerName = theEvent.object;
	}
	else {
		var userRole = "passive";
	}
	
	var action;
	var delta = null;
	var displayDelta = null;
	var message = null;
	switch (theEvent.type) {
		case "attack":
			if (userRole == "active") {
				action = "defended";
				delta = theEvent.subjectDelta;
				displayDelta = theEvent.subject;
			}
			else {
				action = "attacked you";
				delta = theEvent.objectDelta;
				displayDelta = theEvent.object;
			}
			break;
		case "landGrab":
			if (userRole == "active") {
				action = "had land grabbed";
				delta = theEvent.subjectDelta;
				displayDelta = theEvent.subject;
			}
			else {
				action = "land grabbed you";
				delta = theEvent.objectDelta;
				displayDelta = theEvent.object;
			}
			break;
		case "gift":
			if (userRole == "active") {
				action = "received";
				delta = theEvent.subjectDelta;
				displayDelta = theEvent.subject;
			}
			else {
				action = "gave you";
				delta = theEvent.objectDelta;
				displayDelta = theEvent.object;
			}
			break;
		case "message":
			if (userRole == "active") {
				action = "was told";
			}
			else {
				action = "said";
			}
			message = theEvent.message;
			break;
		case "levelup":
			playerName = Brimstone.player.name
			action = "levelled up";
			message = this.message;
			break;
		case "fireball":
			if (userRole == "active") {
				action = "was fireballed";
			}
			else {
				action = "fireballed you";
			}
			delta = this.objectDelta;
			displayDelta = theEvent.object;
	}
	this.displayName = playerName;
	this.id = theEvent.id;
	this.eventDescription = action;
	this.displayDeltaName = displayDelta;
	this.delta = delta;
	this.message = message;
}

brimstoneEventDisplayFields.dataMap = {
	tableName : "eventDisplayFields",
	primaryKey : ["eventId"],
	primaryKeyLookup : {"eventId" : true},
	fieldMappings : {
		"eventId" : {member: "eventId", type : "integer", fieldName : "eventId"},
		"displayName" : {member: "displayName", type : "string", fieldName : "displayName"},
		"eventDescription" : {member: "eventDescription", type : "string", fieldName : "eventDescription"},
		"displayDeltaName" : {member: "displayDeltaName", type : "string", fieldName : "displayDelta"},
	}
}

function brimstoneEventDisplayFields(theEvent) {
	this.theEvent = theEvent;
	this.calculateDisplayFields(theEvent);
}

function brimstoneEvent(id) {
	if (id) {
		this.id = id;
	}
	this.timestamp = getTimeStamp();
	this.subject = null;
	this.object = null;
	this.type = null;
	this.message = null;
	this.subjectDelta = null;
	this.objectDelta = null;
	Brimstone.events.push(this);
}

brimstoneEvent.prototype.__defineGetter__("id", function() {
	return this._id;
});

brimstoneEvent.prototype.__defineSetter__("id", function(id) {
	Brimstone.eventsById[id] = this;
	this._id = id;
});

brimstoneEvent.dataMap = {
	tableName : "events",
	primaryKey : ["id"],
	primaryKeyLookup : {"id" : true},
	fieldMappings : {
		"id" : {member: "id", type : "autoincrement", fieldName : "id"},
		"timestamp" : {member: "timestamp", type : "string", fieldName : "timestamp"},
		"subject" : {member: "subject", type : "string", fieldName : "subject"},
		"object" : {member: "object", type : "string", fieldName : "object"},
		"type" : {member: "type", type : "string", fieldName : "type"},
		"message" : {member: "message", type : "string", fieldName : "message"},
		"subjectDelta" : {member: "subjectDelta", type : "dependentObject", objectType : eventDelta, pkMap : {"eventId" : "id", "actor" : "subject"}},
		"objectDelta" : {member: "objectDelta", type : "dependentObject", objectType : eventDelta, pkMap : {"eventId" : "id", "actor" : "object"}},
		"displayFields" : {member: "displayFields", type : "calculatedObject", objectType : brimstoneEventDisplayFields, pkMap : {"eventId" : "id"}}		
	}
}

function Alliance(url, tag) {
	this.url = url;
	this.tag = tag;
}

Alliance.dataMap = {
	tableName : "alliances",
	primaryKey : ["url"],
	primaryKeyLookup : {"url" : true},
	fieldMappings : {
		"url" : {member: "url", type : "string", fieldName : "url"},
		"tag" : {member: "tag", type : "string", fieldName : "tag"}
	}
}

function BrimstoneParameter(name, value, faction) {
	this.parameter = name;
	this.value = value;
	this.faction = faction;
}

BrimstoneParameter.dataMap = {
	tableName : "parameters",
	primaryKey : ["parameter"],
	primaryKeyLookup : {"parameter" : true},
	fieldMappings : {
		"parameter" : {member: "parameter", type : "string", fieldName : "parameter"},
		"value" : {member: "value", type : "string", fieldName : "value"}
	}
}

var BrimstoneData = {
	xuluData : null,
	xuluDataSettings : {
		currentVersion : 7,
	},
		
	initialise : function() {
			var dataDir = Components.classes["@mozilla.org/file/directory_service;1"]
                   .getService(Components.interfaces.nsIProperties)
                   .get("ProfD", Components.interfaces.nsILocalFile);
		BrimstoneData.dirPermissions = dataDir.permissions;
		BrimstoneData.filePermissions = dataDir.permissions & 0666;			
			dataDir.append("brimstonedata");
		if (!dataDir.exists()) {
		   dataDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, BrimstoneData.dirPermissions);
		}
		this.dataPath = dataDir.path;

		var scriptsHandle = getFileHandle(Brimstone.installedPath);
		scriptsHandle.appendRelativePath("sql");
		
		BrimstoneData.xuluData = new XuluData(BrimstoneData.xuluDataSettings, dataDir.path, scriptsHandle.path);

		BrimstoneData.playerDataStore = BrimstoneData.xuluData.addDataStore(playerInfo);
		BrimstoneData.loadPlayer = function(playerName, DBConn){
			if (Brimstone.players[playerName]) {
				return Brimstone.players[playerName];
			} else {
				var options = {
					whereClause : "WHERE name=?1",
					whereValues : [{type : "string", value : playerName}]
				}
				var matchedPlayers = BrimstoneData.playerDataStore.loadObjects(options, DBConn);
				if (matchedPlayers.length == 1) {
					Brimstone.players[playerName] = matchedPlayers[0];
					return matchedPlayers[0];
				} else {
					return null;
				}		
			}
		};
		
		BrimstoneData.playerCalculationsDataStore = BrimstoneData.xuluData.addDataStore(playerCalculations);

		BrimstoneData.allianceDataStore = BrimstoneData.xuluData.addDataStore(Alliance);
		BrimstoneData.loadAlliances = function(options, DBConn){
			return BrimstoneData.allianceDataStore.loadObjects(options, DBConn);
		}

		BrimstoneData.parametersDataStore = BrimstoneData.xuluData.addDataStore(BrimstoneParameter);

/*			BrimstoneData.loadParameters = function(options, DBConn){
			if (!options) {
				options = {}
			}
			if (!options.returnType) {
				options.returnType = "indexedByPrimaryKey";
			}
			return BrimstoneData.parametersDataStore.loadObjects(options, DBConn);
		} */

		BrimstoneData.loadParameters = function(parameterNames, factionDependent, DBConn){
			var options = {
				returnType : "indexedByPrimaryKey"
			}
			if (factionDependent) {
				var returnObj = {facebook:{}, bebo:{}};
				var parameterList = "('facebook" + parameterNames.join("', 'facebook") + "')";
				options.whereClause = "WHERE parameter IN " + parameterList;
				var tempParameters = BrimstoneData.parametersDataStore.loadObjects(options, DBConn);
				for (var i = 0, l = parameterNames.length; i < l; i++) {
					var parameterName = parameterNames[i];
					var fullName = "facebook" + parameterName;
					if (tempParameters[fullName]) {
						returnObj.facebook[parameterName] = tempParameters[fullName];							
					} else {
						returnObj.facebook[parameterName] = new BrimstoneParameter(fullName);
					}
				}
				var parameterList = "('bebo" + parameterNames.join("', 'bebo") + "')";
				options.whereClause = "WHERE parameter IN " + parameterList;
				var tempParameters = BrimstoneData.parametersDataStore.loadObjects(options, DBConn);
				for (var i = 0, l = parameterNames.length; i < l; i++) {
					var parameterName = parameterNames[i];
					var fullName = "bebo" + parameterName;
					if (tempParameters[fullName]) {
						returnObj.bebo[parameterName] = tempParameters[fullName];							
					} else {
						returnObj.bebo[parameterName] = new BrimstoneParameter(fullName);
					}
				}
				return returnObj;
			}
			else {
				var parameterList = "('" + parameterNames.join("','") + "')"
				options.whereClause = "WHERE parameter IN " + parameterList;
				var tempParameters = BrimstoneData.parametersDataStore.loadObjects(options, DBConn);
				for (var i = 0, l = parameterNames.length; i < l; i++) {
					var parameterName = parameterNames[i];
					if (!tempParameters[parameterName]) {
						tempParameters[parameterName] = new BrimstoneParameter(parameterName);
					}
				}
				return tempParameters;
			}
		}
		
		BrimstoneData.loadHandlerParameters = function(DBConn){
			return BrimstoneData.loadParameters(["lastmessagelog"], true, DBConn);
		}

		BrimstoneData.eventsDataStore = BrimstoneData.xuluData.addDataStore(brimstoneEvent);
		BrimstoneData.loadEvents = function(playerName, DBConn){
			var startDate = new Date();
			jsdump("Started loading events: " + startDate);
			if (!DBConn)
			{
				DBConn = BrimstoneData.xuluData.newConnection();
			}
//				BrimstoneData.xuluData.beginTransaction(DBConn)
			var options = {
				whereClause : "WHERE subject = ?1 OR object = ?2",
				whereValues : [
					{type : "string", value : playerName},
					{type : "string", value : playerName}
				]
			}
			var events =  BrimstoneData.eventsDataStore.loadObjects(options, DBConn);
			var endDate = new Date()
			jsdump("Finished loading events: " + endDate);
			jsdump("Time taken: " + (endDate - startDate));
			return events;
		}
		BrimstoneData.eventDeltaDataStore = BrimstoneData.xuluData.addDataStore(eventDelta);
		BrimstoneData.loadEventDelta = function(DBConn, id, actor){
			var options = {
				whereClause : "WHERE eventId=?1 AND actor=?2",
				whereValues : [{
					type: "integer",
					value: id
				}, 
				{
					type: "string",
					value: actor
				}]
			}
			return BrimstoneData.eventDeltaDataStore.loadObjects(options, DBConn)[0];
		}
		BrimstoneData.brimstoneEventDisplayFieldsDataStore = BrimstoneData.xuluData.addDataStore(brimstoneEventDisplayFields);
//			var alliance = new Alliance("abc", "def");
//			alliance.save();
	},
	
	close : function() {
		if (BrimstoneData.DBConn) {
			if (BrimstoneData.DBConn.connectionReady) {
				// Apparently, despite MDC's comments to the contrary, close() doesn't exist
	//			BrimstoneData.DBConn.close();
			}
			BrimstoneData.DBConn = null;
		}
		BrimstoneData.xuluData.terminate();
	},


	cleanData : function() {
		BrimstoneData.xuluData.cleanData();
	},
	
	savePlayer : function(player, DBConn) {
		if (player.name) {
			player.timestamp = getTimeStamp();
			player.save(DBConn);
		}
	},
	
	loadPlayer : function(playerName, DBConn) {
	},
	
	saveEvent : function(event, DBConn) {
			if (XULU_APP_RELEASE_TYPE == "development" && event.type == "unknown") {
				alert("Unknown event");
			}
			event.save(DBConn);
	}
}

var BrimstoneBrowser = {
	
	parentWindow : null,
	tabbedBrowser : null,
	listeners : {},
	
	initialise : function() {
		try {
			BrimstoneBrowser.parentWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
			                   .getInterface(Components.interfaces.nsIWebNavigation)
			                   .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
			                   .rootTreeItem
			                   .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
			                   .getInterface(Components.interfaces.nsIDOMWindow);
			BrimstoneBrowser.tabbedBrowser =  BrimstoneVariant.BrimstoneBrowser.getTabbedBrowser();
			for (var i = 0, l = BrimstoneBrowser.tabbedBrowser.browsers.length; i < l; i++) {
				var browser = BrimstoneBrowser.tabbedBrowser.getBrowserAtIndex(i);
				browser.addEventListener("load", BrimstoneBrowser, true);
				browser.addEventListener("beforeunload", BrimstoneBrowser, true);
				browser.addEventListener("click", BrimstoneBrowser, true);
				browser.addProgressListener(BrimstoneListener);
//				jsdump("Adding listener for tab " + i);
			}
			// During initialisation
			var container = BrimstoneBrowser.tabbedBrowser.tabContainer;
			container.addEventListener("TabOpen", BrimstoneBrowser.tabAdded, false);
//			BrimstoneBrowser.parentBrowser.addEventListener("load", function(event) {BrimstoneBrowser.eventListener("load", event);}, true);
		} catch(e) {
			BrimstoneErrorHandler.handleError("initialising browser", e);
		}
	},
	
	onClose : function() {
//		jsdump("onClose")
		var container = BrimstoneBrowser.tabbedBrowser.tabContainer;
		container.removeEventListener("TabOpen", BrimstoneBrowser.tabAdded, false);
		for (var i = 0, l = BrimstoneBrowser.tabbedBrowser.browsers.length; i < l; i++) {
			var browser = BrimstoneBrowser.tabbedBrowser.getBrowserAtIndex(i);
			try {
//				jsdump("Removing listener for tab " + i);
    			browser.removeEventListener("load", BrimstoneBrowser, true);
				browser.removeEventListener("beforeunload", BrimstoneBrowser, true);
				browser.removeEventListener("click", BrimstoneBrowser, true);
				browser.removeProgressListener(BrimstoneListener);
			} catch(e) {
				Components.utils.reportError(e);
			}
		}
//		BrimstoneBrowser.parentBrowser.removeEventListener("load", function(event) {BrimstoneBrowser.eventListener("load", event);}, true);
	},
	
	tabAdded : function(event) {
		try {
			jsdump("Tab added");
			var browser = event.target.linkedBrowser;
			browser.addEventListener("load", BrimstoneBrowser, true);			
			browser.addEventListener("click", BrimstoneBrowser, true);			
			browser.addProgressListener(BrimstoneListener);
		} catch (e) {
			XuluErrorHandler.handleError("TabOpen event", e);
		}
	},
	
	urlsAreEquivalent : function(url1, url2) {
		var url1Length = url1.length;
		var url2Length = url2.length;
		if (url1Length == url2Length) {
			return url1 == url2;
		} else if (url1Length < url2Length) {
			return url1 + "/" == url2;
		} else {
			return url1 == url2 + "/";
		}
	},
	
	renameTab : function(url, name) {
		var l = BrimstoneBrowser.tabbedBrowser.browsers.length
		for (var i = 0; i < l; i++) {
			if (BrimstoneBrowser.urlsAreEquivalent(BrimstoneBrowser.tabbedBrowser.browsers[i].currentURI.spec, url)) {
				var tab = BrimstoneBrowser.tabbedBrowser.tabContainer.childNodes[i];
				tab.label = name;
				break;
			}
		}
		
	},
	
	loadPage : function(url) {
		var l = BrimstoneBrowser.tabbedBrowser.browsers.length
		for (var i = 0; i < l; i++) {
			if (BrimstoneBrowser.urlsAreEquivalent(BrimstoneBrowser.tabbedBrowser.browsers[i].currentURI.spec, url)) {
				var tab = BrimstoneBrowser.tabbedBrowser.tabContainer.childNodes[i];
				tab.linkedBrowser.reload();
				break;
			}
		}
		if (!tab) {
			var tab = BrimstoneBrowser.tabbedBrowser.addTab(url);
		}
		// Set selected browser;
		BrimstoneBrowser.tabbedBrowser.selectedTab = tab;
	},
	
	addEventListener : function(eventName, listener, x) {
		if (!BrimstoneBrowser.listeners[eventName]) {
			BrimstoneBrowser.listeners[eventName] = {};
		}
		BrimstoneBrowser.listeners[eventName][listener] = listener;
	},
	
	removeEventListener : function(eventName, listener, x) {
		if (BrimstoneBrowser.listeners[eventName] && BrimstoneBrowser.listeners[eventName][listener]) {
			delete BrimstoneBrowser.listeners[eventName][listener];
		}
	},
	
	handleEvent : function(event) {
		if (typeof(BrimstoneBrowser) != "undefined"){
//			jsdump("Event: " + event.type);
//			jsdump("name: " + typeof(event.currentTarget.nodeName));
			for (var listener in BrimstoneBrowser.listeners[event.type]) {
//							jsdump("Sending event to " + listener);
				BrimstoneBrowser.listeners[event.type][listener](event);
			}
		} else {
			event.currentTarget.removeEventListener(event.type, this, true);
			throw new Error("Didn't remove BrimstoneBrowser as event listener for " + event.type);
		}
	},
}

var BrimstoneErrorHandler = XuluErrorHandler;

var BrimstonePrefObsever =
{
	brimstonePrefs : null,
	
	register: function()
	{
		BrimstonePrefObsever.brimstonePrefs = Components.classes["@mozilla.org/preferences-service;1"].
		                    getService(Components.interfaces.nsIPrefService);
		BrimstonePrefObsever.brimstonePrefs = BrimstonePrefObsever.brimstonePrefs.getBranch("extensions.brimstone.");
		BrimstonePrefObsever.brimstonePrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
		BrimstonePrefObsever.brimstonePrefs.addObserver("", this, false);
	},

	unregister: function()
	{
		BrimstonePrefObsever.brimstonePrefs.removeObserver("", this);
	},

	observe: function(aSubject, aTopic, aData)
	{
		if(aTopic != "nsPref:changed") return;
		// aSubject is the nsIPrefBranch we're observing (after appropriate QI)
		// aData is the name of the pref that's been changed (relative to aSubject)
		switch (aData) {
			case "player.facebookName":
			case "player.beboName":
//				alert(aData);
				// extensions.myextension.pref1 was changed
				Brimstone.playerChanged();
				break;
			case "hideBlankValues" :
				if (BrimstoneOptions.hideBlankValues) {
					BrimstoneUI.updatePlayerInfo();
					BrimstoneUI.updateTargetInfo();
				} else {
					BrimstoneUI.playerUI.showValues(true);
					BrimstoneUI.targetUI.showValues(true);
				}
				break;
			case "easyTargetTolerance" :
			case "improbableTargetTolerance" :
				BrimstoneUI.invalidateDefenceStyling();
				break;
			case "noMaxWidth" :
				if (BrimstoneOptions.noMaxWidth) {
					BrimstoneUI.setSidebarMaxWidth("none");
				} else {
					BrimstoneUI.resetSidebarMaxWidth();
				}
				break;
		}
	}
}

window.addEventListener('load', Brimstone.onLoad, false);
window.addEventListener('unload', Brimstone.onClose, false);