function dataStore(xuluData, DBpath, classObj) {
	this.xuluData = xuluData;
	this.DBpath = DBpath;
	this.objectConstructor = classObj;
	this.dataMap = this.prepareDataMap(classObj.dataMap);
	this.transactionOwner = false;
	this.transactionConn = null;
	this.eventListeners = {};
}

dataStore.prototype.createTempDataStore = function(tempTableName, options, DBConn){
//	return this;
	var tempDataMap = {};
	for (var memberName in this.objectConstructor.dataMap) {
		if (typeof(this.objectConstructor.dataMap[memberName] != "function")) {
			tempDataMap[memberName] = this.objectConstructor.dataMap[memberName];
		}
	}
	var tempDataStore = new dataStore(this.xuluData, this.DBpath, this.objectConstructor);
	tempDataStore.dataMap = tempDataStore.prepareDataMap(tempDataMap, tempTableName);
	
	var whereClause = "";
	var whereValues = null;
	var orderClause = "";
	var limitClause = "";
	if (options) {
		if (options.whereClause) {
			// do stuff
			whereClause = options.whereClause;
			var whereValues = options.whereValues;
		}
		if (options.limitClause) {
			limitClause = options.limitClause;
		}
		if (options.orderClause) {
			orderClause = options.orderClause;
		}
	}
	this.xuluData.executeStatement("DROP TABLE IF EXISTS {0}".format(tempTableName), null, DBConn);
	
	this.xuluData.executeStatement("CREATE TEMP TABLE {0} AS {1}".format(tempTableName, this.dataMap._selectClauseTemplate.format(whereClause, orderClause, limitClause)), whereValues, DBConn);
	return tempDataStore;
}

dataStore.prototype.prepareDataMap = function(dataMap, tempTableName) {
	if (tempTableName) {
		var tableName = tempTableName;
	} else {
		var tableName = dataMap.tableName;
	}
	dataMap._primaryKeyLookup = {};
	dataMap._orderedNormalBoundFields = [];
	dataMap._orderedNormalBoundFieldsExcPrimaryKey = [];
	dataMap._dependentObjectFields = [];
	dataMap._flattenedObjectFieldMaps = [];
	dataMap._calculatedObjectFieldMaps = [];
	
	for (var i = 0, l= dataMap.primaryKey.length; i < l; i++) {
		var fieldName = dataMap.primaryKey[i]
		dataMap._primaryKeyLookup[fieldName] = i;
		dataMap._orderedNormalBoundFields.push(dataMap.fieldMappings[fieldName])
		if (dataMap.fieldMappings[fieldName].type == "autoincrement") {
			dataMap._autoIncrementField = fieldMapping;		
		}
	}
	
	for (var keyName in dataMap.fieldMappings) {
		var fieldMapping = dataMap.fieldMappings[keyName];
		if (!fieldMapping.member) {
			fieldMapping.member = keyName;
		}
		if (!fieldMapping.fieldName) {
			fieldMapping.fieldName = keyName;
		}
		switch (fieldMapping.type) {
			case "dependentObject" :
				dataMap._dependentObjectFields.push(fieldMapping);
				break;
			case "flattenedObject" :
				dataMap._flattenedObjectFieldMaps.push(fieldMapping);
				break;
			case "calculatedObject" :
				dataMap._calculatedObjectFieldMaps.push(fieldMapping);
				break;
			default :
				if (dataMap._primaryKeyLookup[keyName] == null) {
					dataMap._orderedNormalBoundFields.push(dataMap.fieldMappings[keyName])
					dataMap._orderedNormalBoundFieldsExcPrimaryKey.push(dataMap.fieldMappings[keyName])
				}
		}
	}

	dataMap._dependentAndCalculatedObjectFieldMaps = dataMap._dependentObjectFields.concat(dataMap._calculatedObjectFieldMaps);

	dataMap._flattenedObjectFields = [];
	
	for (var i = 0, l = dataMap._flattenedObjectFieldMaps.length; i < l; i++) {
		var foreignFieldMappings = dataMap._flattenedObjectFieldMaps[i].objectType.dataMap.fieldMappings;
		for (var fieldName in foreignFieldMappings) {
			var fieldMap = foreignFieldMappings[fieldName];
			dataMap._flattenedObjectFields.push(fieldMap);
			fieldMap._flattenedSelectIndex = dataMap._orderedNormalBoundFields.length + dataMap._flattenedObjectFields.length - 1;
		}
	}
	
	// Create Select Clause Template
	var selectItems = this.generateSelectItems(dataMap._orderedNormalBoundFields.concat(dataMap._flattenedObjectFields));
	dataMap._selectClauseTemplate = "SELECT {0} FROM {1}".format(selectItems, tableName) + " {0} {1} {2}";
//	jsdump("Select unique template: " + dataMap._selectClauseTemplate)

	// Create Select Unique Clause
	var whereClause = this.generateWhereClause(dataMap.primaryKey);
	dataMap._selectUniqueClause = dataMap._selectClauseTemplate.format(whereClause, "", "");
//	jsdump("Select unique clause: " + dataMap._selectUniqueClause)
	
	var valuePlaceHolders = this.generateValuePlaceHolders(dataMap._orderedNormalBoundFields.concat(dataMap._flattenedObjectFields));
	dataMap._insertClause = "INSERT INTO {0} ({1}) VALUES ({2})".format(tableName, selectItems, valuePlaceHolders);
//	jsdump("insert clause: " + dataMap._insertClause);

	var updatePairs = this.generateUpdatePairs(dataMap._orderedNormalBoundFieldsExcPrimaryKey.concat(dataMap._flattenedObjectFields), dataMap.primaryKey.length + 1);
	dataMap._updateClause = "UPDATE {0} SET {1} {2}".format(tableName, updatePairs, whereClause)
//	jsdump("update clause: " + dataMap._updateClause);
	return dataMap;
}

dataStore.prototype.newConnection = function() {
//	jsdump("datapath: " + this.DBpath);
	return this.xuluData.newConnection();
}

dataStore.prototype.generateUpdatePairs = function(keys, startIndex) {
	if (keys.length == 0) {
		return "";
	} else {
		var selectItems = "";
		for (var i = 0, l = keys.length; i < l; i++) {
			if (keys[i].fieldName) {
				var keyName = keys[i].fieldName;
			} else {
				var keyName = keys[i];			
			}
			if (selectItems.length > 0) {
				selectItems += ", ";
			}
			selectItems += "{0}=?{1}".format(keyName, i + startIndex);
		}
		return selectItems;
	}	
}

dataStore.prototype.generateSelectItems = function(keys) {
	if (keys.length == 0) {
		return "";
	} else {
		var selectItems = "";
		for (var i = 0, l = keys.length; i < l; i++) {
			if (keys[i].fieldName) {
				var keyName = keys[i].fieldName;
			} else {
				var keyName = keys[i];			
			}
			if (selectItems.length > 0) {
				selectItems += ", ";
			}
			selectItems += keyName;
		}
		return selectItems;
	}	
}

dataStore.prototype.generateValuePlaceHolders = function(keys) {
	if (keys.length == 0) {
		return "";
	} else {
		var selectItems = "";
		for (var i = 0, l = keys.length; i < l; i++) {
			if (selectItems.length > 0) {
				selectItems += ", ";
			}
			selectItems += "?" + (i + 1);
		}
		return selectItems;
	}	
}

dataStore.prototype.generateWhereClause = function(keys) {
	if (keys.length == 0) {
		return "";
	} else {
		var whereClause = "";
		for (var i = 0, l = keys.length; i < l; i++) {
			if (keys[i].fieldName) {
				var keyName = keys[i].fieldName;
			} else {
				var keyName = keys[i];			
			}
			if (whereClause.length > 0) {
				whereClause += " AND ";
			}
			whereClause += "{0} = ?{1}".format(keyName, i+1);
		}
		return "WHERE " + whereClause;
	}
}

dataStore.prototype.saveObject = function(object, DBConn) {
	try {
//		jsdump("Saving Object")
		var autoIncrementField = null;
		var action = null;
		var whereClause = "";
		var insertFields = "(";
		var insertValues = "(";
		var orderedFields = [];
		var updateFields = "";
		
		for (var i = 0, l = this.dataMap.primaryKey.length; i < l; i++) {
			var keyName = this.dataMap.primaryKey[i];
			if (!object[keyName]) {
				if (this.dataMap.fieldMappings[keyName].type == "autoincrement") {
					autoIncrementField = keyName;
				}
				else {
					// primary key fields not set.  Should probably return an error
					throw new Error("Primary key field '{0}' is not defined".format(this.dataMap.primaryKey[i]));
				}
			}
		}
		if (!DBConn) {
			DBConn = this.newConnection();
//			XuluData.beginTransaction(DBConn);
//			var beganTransaction = true;
		}

		if (autoIncrementField) {
			action = "insert";
		} else {
			var statement = DBConn.createStatement(this.dataMap._selectUniqueClause);//
		
			for (var i = 0, l = this.dataMap.primaryKey.length; i < l; i++) {
				var keyName = this.dataMap.primaryKey[i];
				this.bindValue(statement, this.dataMap.fieldMappings[keyName].type, i, object[keyName]);
			}
	
			if (statement.executeStep()) {
				// row exists
				action = "update";
			} else {
				//row doesn't exist
				action = "insert";
			}
			statement.reset();
		}
		if (action == "update") {
//			jsdump("UPDATE");
			statement = DBConn.createStatement(this.dataMap._updateClause);
		} else {
			action = "insert";
//			jsdump("INSERT");
			try {
				statement = DBConn.createStatement(this.dataMap._insertClause);			
			} catch (e) {
				// Probably a no such collation sequence error.
				throw e; // Put break point here.
			}
		}
		for (var i = 0, l = this.dataMap._orderedNormalBoundFields.length; i < l; i++) {
			var keyName = this.dataMap._orderedNormalBoundFields[i].member;
//			jsdump("keyName: " + keyName)
			this.bindValue(statement, this.dataMap.fieldMappings[keyName].type, i, object[keyName]);
		}
		
		for (var i = 0, l = this.dataMap._flattenedObjectFieldMaps.length; i < l; i++) {
			var fieldMap = this.dataMap._flattenedObjectFieldMaps[i];
			var subObject = object[fieldMap.member]
			if (subObject) {
				var subFieldMappings = fieldMap.objectType.dataMap.fieldMappings;
				for (var fieldName in subFieldMappings) {
					var subFieldMap = subFieldMappings[fieldName];
					this.bindValue(statement, subFieldMap.type, subFieldMap._flattenedSelectIndex, subObject[subFieldMap.member]);
				}
			}
		}
		statement.execute();

		if (action == "insert" && autoIncrementField) {
			object[autoIncrementField] = DBConn.lastInsertRowID;
		}
		
		for (var i=0, l1=this.dataMap._dependentAndCalculatedObjectFieldMaps.length; i < l1; i++) {
			var fieldMap = this.dataMap._dependentAndCalculatedObjectFieldMaps[i];
			var memberName = fieldMap.member;
			if (object[memberName]) {
				var dependentObject = object[memberName];
				var dependentDataMap = fieldMap.objectType.dataMap;
				for (var j=0, l2 = dependentDataMap.primaryKey.length; j < l2; j++) {
					var keyName = dependentDataMap.primaryKey[j];
					dependentObject[keyName] = object[fieldMap.pkMap[keyName]];
				}
				dependentObject.save(DBConn);
			}
		}
		
//		if (beganTransaction) {
//			XuluData.commitTransaction(DBConn);
//		}
		this.broadcastEvent(action, DBConn);

		if (action == "insert" && autoIncrementField) {
			return DBConn.lastInsertRowID;
		}
	} catch (e) {
		if (DBConn && DBConn.lastError && DBConn.lastErrorString != "not an error") {
			var newError = Error(DBConn.lastErrorString);
			newError.DBError = e;
			throw newError;
		} else {
			throw e;
		}
	} finally {
		if (statement) {
			statement.reset();
		}
	} 
	
}

dataStore.prototype.bindValue = function(statement, type, index, value) {
	this.xuluData.bindValue(statement, type, index, value)
}

dataStore.prototype.getValue = function(statement, type, index) {
	if (statement.getIsNull(index)) {
		return null;
	} else {
		switch (type) {
			case "autoincrement":
			case "integer":
				return statement.getInt32(index);
				break;
			case "string":
				return statement.getString(index);
				break;
			case "float":
				return statement.getDouble(index);
				break;
			case "dateasinteger":
				return new Date(statement.getInt64(index));
				break;
		}
	}	
}

dataStore.prototype.getCount = function(options, DBConn){
	try {
		var whereClause = "";
		if (options) {
			if (options.whereClause) {
				// do stuff
				whereClause = options.whereClause;
				var whereValues = options.whereValues;
			}
		}
		if (!DBConn) {
			DBConn = this.newConnection();
		}
		var statement = DBConn.createStatement("SELECT COUNT(*) FROM {0} {1}".format(this.dataMap.tableName, whereClause));//
		if (whereValues) {
			for (var i in whereValues) {
				this.bindValue(statement, whereValues[i].type, i, whereValues[i].value);
			}
		}
		if (statement.executeStep()) {
			return statement.getInt32(0);
		}
	} catch (e) {
		if (DBConn && DBConn.lastError && DBConn.lastErrorString != "not an error") {
			var newError = Error(DBConn.lastErrorString);
			newError.DBError = e;
			throw newError;
		} else {
			throw e;
		}
	} finally {
		if (statement) {
			statement.reset();
		}
	}		
}

dataStore.prototype.loadObjects = function(options, DBConn) {
	try {
		var whereClause = "";
		var orderClause = "";
		var limitClause = "";
		var returnType = "array";
		if (options) {
			if (options.whereClause) {
				// do stuff
				whereClause = options.whereClause;
				var whereValues = options.whereValues;
			}
			if (options.limitClause) {
				limitClause = options.limitClause;
			}
			if (options.orderClause) {
				orderClause = options.orderClause;
			}
			
			if (options.returnType) {
				switch (options.returnType) {
					case "indexedByPrimaryKey" :
						returnType = "indexedByPrimaryKey"
						break;
				}
			} 
		}
		switch (returnType){
			case "array" : 
				var objects = [];
				break;
			case "indexedByPrimaryKey" :
				var objects = {};
				break;
		}
		if (!DBConn) {
			DBConn = this.newConnection();
		}

//		jsdump("select statement: " + "SELECT {0} FROM {1} {2}".format(fields, this.dataMap.tableName, whereClause))
		var statement = DBConn.createStatement(this.dataMap._selectClauseTemplate.format(whereClause, orderClause, limitClause));//
		if (whereValues) {
			for (var i in whereValues) {
				this.bindValue(statement, whereValues[i].type, i, whereValues[i].value);
			}
		}
//		statement.bindStringParameter(0, playerName);
//		statement.bindStringParameter(1, getTimeStamp(-1));
		while (statement.executeStep()) {
			var object = new this.objectConstructor();
			for (var i=0, l=this.dataMap._orderedNormalBoundFields.length; i < l; i++) {
				var keyName = this.dataMap._orderedNormalBoundFields[i].member;
//				jsdump("keyName: {0} type: {1}".format(keyName, this.dataMap.fieldMappings[keyName].type))
				object[keyName] = this.getValue(statement, this.dataMap.fieldMappings[keyName].type, i);		
			}
			for (var i = 0, l = this.dataMap._dependentObjectFields.length; i < l; i++) {
				var fieldMap = this.dataMap._dependentObjectFields[i];
				var whereValues = [];
				var dependentDataMap = fieldMap.objectType.dataMap;
				for (var i=0, l = dependentDataMap.primaryKey.length; i < l; i++) {
					var keyName = dependentDataMap.primaryKey[i];
					var keyMap = dependentDataMap.fieldMappings[keyName];
					whereValues.push({type : keyMap.type, value : object[fieldMap.pkMap[keyName]]})
				}
				var options = {
					whereClause : this.generateWhereClause(dependentDataMap.primaryKey),
					whereValues : whereValues
				}
				object[fieldMap.member] = fieldMap.objectType.dataStore.loadObjects(options, DBConn)[0];
			}
			
			for (var i = 0, l = this.dataMap._flattenedObjectFieldMaps.length; i < l; i++) {
				var fieldMap = this.dataMap._flattenedObjectFieldMaps[i];
				var foreignFieldMappings = fieldMap.objectType.dataMap.fieldMappings;
				var foreignFieldMap = foreignFieldMappings[fieldMap.createIfDefined];
				if (!statement.getIsNull(foreignFieldMap._flattenedSelectIndex)) {
					var foreignObject = new fieldMap.objectType();
					object[fieldMap.member] = foreignObject;
					for (var foreignFieldKey in foreignFieldMappings) {
						var foreignFieldMap = foreignFieldMappings[foreignFieldKey];
						foreignObject[foreignFieldMap.member] = this.getValue(statement, foreignFieldMap.type, foreignFieldMap._flattenedSelectIndex);
					}
				}
				var a = 1;
/*				var whereValues = [];
				var dependentDataMap = fieldMap.objectType.dataMap;
				for (var i=0, l = dependentDataMap.primaryKey.length; i < l; i++) {
					var keyName = dependentDataMap.primaryKey[i];
					var keyMap = dependentDataMap.fieldMappings[keyName];
					whereValues.push({type : keyMap.type, value : object[fieldMap.pkMap[keyName]]})
				}
				var options = {
					whereClause : this.generateWhereClause(dependentDataMap.primaryKey),
					whereValues : whereValues
				}
				object[fieldMap.member] = fieldMap.objectType.dataStore.loadObjects(options, DBConn)[0]; */
			}

			switch (returnType){
				case "array" : 
					objects.push(object);
					break;
				case "indexedByPrimaryKey" :
					objects[object[this.dataMap.primaryKey[0]]] = object;
					break;
			}
		}
		return objects;
	} catch (e) {
		if (DBConn && DBConn.lastError && DBConn.lastErrorString != "not an error") {
			var newError = Error(DBConn.lastErrorString);
			newError.DBError = e;
			throw newError;
		} else {
			throw e;
		}
	} finally {
		if (statement) {
			statement.reset();
		}
	}
}

dataStore.prototype.addEventListener = function(event, listener) {
	this.eventListeners[listener] = listener;
}

dataStore.prototype.removeEventListener = function(event, listener) {
	delete this.eventListeners[listener];
}

dataStore.prototype.broadcastEvent = function(event, DBConn) {
	for (var key in this.eventListeners) {
		var listener = this.eventListeners[key];
		listener.dataStoreEvent(event, DBConn);
	}
}

function XuluData(settings, dataDirPath, scriptsPath){
	this.settings = {
		currentVersion: null,
		dataFilePrefix: "data.sqlite",
		upgradeFilePrefix: "upgrade.sqlite.sql",
		cleanFile: "clean.sqlite.sql",
		tempFileName: "_tempDB"
	};
	
	if (settings) {
		for (var settingName in settings) {
			this.settings[settingName] = settings[settingName];
		}
	}
	
	this.transactionCount = 0;
	this.storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);
	
	
	var dataDir = getFileHandle(dataDirPath);
	var parentDir = dataDir.parent;
	this.dirPermissions = parentDir.permissions;
	this.filePermissions = parentDir.permissions & 0666;
	//		dataDir.append(dataDirName);
	if (!dataDir.exists()) {
		dataDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, this.dirPermissions);
	}
	this.dataDirPath = dataDir.path;
	this.scriptsPath = scriptsPath;
	dataDir.append(this.settings.dataFilePrefix + "." + this.settings.currentVersion);
	this.dataPath = dataDir.path;
	
	this.parentWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor).getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsIDocShellTreeItem).rootTreeItem.QueryInterface(Components.interfaces.nsIInterfaceRequestor).getInterface(Components.interfaces.nsIDOMWindow);
	if (this.parentWindow.XuluData) {
		if (this.parentWindow.XuluData[this.dataPath]) {
			jsdump("Waiting for last instance to terminate properly");
			var d = new Date() //today's date
			var Amount = 1000;
			while (1) {
				var mill = new Date() // Date Now
				var diff = mill - d //difference in milliseconds
				if (diff > Amount) {
					break;
				}
			}
		}
		try {
			delete (this.parentWindow.XuluData[this.dataPath]);
		} 
		catch (e) {
		}
	}
	else {
		this.parentWindow.XuluData = {}
	}
	this.upgrade();
}

XuluData.prototype.terminate = function() {
	this.parentWindow.XuluData[this.dataPath] = "terminating";
	var parentWindow = this.parentWindow;
	var dataPath = this.dataPath;
	this.parentWindow.setTimeout(function() {delete(parentWindow.XuluData[dataPath])}, 1000);
}
	
XuluData.prototype.newConnection = function() {
	return this.storageService.openDatabase(getFileHandle(this.dataPath));
};
	
XuluData.prototype.addDataStore = function(classObj) {
	classObj.dataStore = new dataStore(this, this.dataPath, classObj);
	classObj.prototype.dataStore = classObj.dataStore;
	classObj.prototype.save = function(DBConn) {
		this.dataStore.saveObject(this, DBConn);
	}
	return classObj.prototype.dataStore;
}
	
XuluData.prototype.getInstalledVersion = function(installedDir) {
	var installedVersion = 0;
	var file = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
	for (var i = this.settings.currentVersion; i > 0; i--) {
		file.initWithPath(this.dataDirPath);
		file.append(this.settings.dataFilePrefix + "." + i);
		if (file.exists()) {
			installedVersion = i;
			break;
		}
	}
	return installedVersion;
}
	
XuluData.prototype.upgrade = function() {
	var dataDir = getFileHandle(this.dataDirPath);
	var installedVersion = this.getInstalledVersion(dataDir);
//		jsdump("Installed Version: " + installedVersion);
	if (installedVersion  == this.settings.currentVersion) {
		return;
	}
	try {
		var tempDBFile = getFileHandle(this.dataDirPath);
		tempDBFile.append(this.settings.tempFileName);
		if (tempDBFile.exists()) {
			tempDBFile.remove(false);
		}
		var file = Components.classes["@mozilla.org/file/local;1"]
	                     .createInstance(Components.interfaces.nsILocalFile);
		if (installedVersion > 0) {
			file.initWithPath(this.dataDirPath);
			file.append(this.settings.dataFilePrefix + "." + installedVersion);
			file.copyTo(dataDir, this.settings.tempFileName);
		} else {
			tempDBFile.create(tempDBFile.NORMAL_FILE_TYPE, this.filePermissions);
		}
		
		this.upgradeTempDB(installedVersion);
		
		// There seems to be a problem with the DB connection locking the temp file and
		// taking time to release it.  Pause for a bit and then continue.
/*	    var d = new Date() //today's date
	    var Amount = 1000;
		while (1)
	    {
		    var mill=new Date() // Date Now
		    var diff = mill-d //difference in milliseconds
		    if( diff > Amount ) {break;}
		}
		var why = tempDBFile.path; */
		tempDBFile.moveTo(dataDir, this.settings.dataFilePrefix + "." + this.settings.currentVersion);
	} catch (e) {
		alert("Upgrade of database failed.  This is bad!!\n" + e.message);
		throw e;
	}
}

XuluData.prototype.upgradeTempDB = function(installedVersion) {
	var tempDBFile = getFileHandle(this.dataDirPath);
	tempDBFile.append(this.settings.tempFileName);
//			file.initWithPath(Brimstone.installedPath);
//			file.append(this.settings.dataFilePrefix + "." + installedVersion);
	var tempDBConn = this.storageService.openDatabase(tempDBFile);

	try {
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		
		for (var i = installedVersion + 1; i <= this.settings.currentVersion; i++) {
			file.initWithPath(this.scriptsPath);
			file.append(this.settings.upgradeFilePrefix + "." + i);
			// open an input stream from file
			var istream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
			try {
				istream.init(file, 0x01, 0444, 0);
				istream.QueryInterface(Components.interfaces.nsILineInputStream);
				
				// read lines into array
				var line = {}, lines = [], hasmore;
				do {
					hasmore = istream.readLine(line);
					tempDBConn.executeSimpleSQL(line.value);
				}
				while (hasmore);
			}
			finally {
				istream.close();
			}
		}
	} finally {
		tempDBConn.close();
	}
}

XuluData.prototype.cleanData = function() {
	var DBConn = this.newConnection();


	var file = getFileHandle(this.settings.cleanFile);
	// open an input stream from file
	var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
	                        .createInstance(Components.interfaces.nsIFileInputStream);
	try {
		istream.init(file, 0x01, 0444, 0);
		istream.QueryInterface(Components.interfaces.nsILineInputStream);
		
		// read lines into array
		var line = {}, lines = [], hasmore;
		do {
		  hasmore = istream.readLine(line);
		  DBConn.executeSimpleSQL(line.value);
		} while(hasmore);
	} finally {
		istream.close();					
	}				
}
	
XuluData.prototype.beginTransaction = function(DBConn) {
	this.transactionCount++;
	if (!DBConn.transactionInProgress) {
		DBConn.beginTransactionAs(DBConn.TRANSACTION_DEFERRED);
//			jsdump("Transaction started");
	}
}

XuluData.prototype.commitTransaction = function(DBConn) {
//		jsdump("transactionCount: " + XuluData.transactionCount)
	if (--this.transactionCount == 0) {
		DBConn.commitTransaction();
//			jsdump("Transaction ended");
	}	
}

XuluData.prototype.rollbackTransaction = function(DBConn) {
//		jsdump("transactionCount: " + XuluData.transactionCount)
	if (DBConn.transactionInProgress) {
		this.transactionCount--;
		DBConn.rollbackTransaction();
//			jsdump("Transaction ended");
	}	
}
XuluData.prototype.bindValue =  function(statement, type, index, value) {
	if (value != null) {
		switch (type) {
			case "autoincrement":
			case "integer":
				statement.bindInt32Parameter(index, value);
				break;
			case "string":
				statement.bindStringParameter(index, value);
				break;
			case "float":
				statement.bindDoubleParameter(index, value);
				break;
			case "dateasinteger":
				statement.bindInt64Parameter(index, value.valueOf());
				break;
		}
	}
}
	
XuluData.prototype.executeStatement = function(statementString, bindValues, DBConn) {
	try {
		if (!DBConn) {
			DBConn = this.newConnection();
		}
		
		var statement = DBConn.createStatement(statementString);
		if (bindValues) {
			for (var i in bindValues) {
				this.bindValue(statement, bindValues[i].type, i, bindValues[i].value);
			}
		}
		var returnObj = [];
		while (statement.executeStep()) {
			var rowResult = new Array(statement.columnCount);
			for (var i=0, l=statement.columnCount; i < l; i++) {
				rowResult[i] = statement.getString(i);
			}
			returnObj.push(rowResult);
		}
		return returnObj;
	} catch(e) {
		if (DBConn && DBConn.lastError && DBConn.lastErrorString != "not an error") {
			var newError = Error(DBConn.lastErrorString);
			newError.DBError = e;
			throw newError;
		} else {
			throw e;
		}
	} finally {
		if (statement) {
			statement.reset();
		}
	}
}