var gAtomService = Components.classes["@mozilla.org/atom-service;1"].
		              getService(Components.interfaces.nsIAtomService);

function getMember(object, memberString) {
  var members = memberString.split(".");
  var firstPart = object[members[0]];
  if (firstPart && members.length > 1) {
    return getMember(firstPart, members.splice(1).join("."))
  }  else {
  	return firstPart;
  }
}

function setPersistValue(element, attribute){
	var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
	var rdfLocalStoreDS = Components.classes["@mozilla.org/rdf/datasource;1?name=local-store"].getService(Components.interfaces.nsIRDFDataSource);
	var rdfRes = rdfService.GetResource(window.location.href + "#"+element.id);
	var attributeRes = rdfService.GetResource(attribute);
	
	if (rdfLocalStoreDS.hasArcOut(rdfRes, attributeRes)) {
		var attributeValue = rdfLocalStoreDS.GetTarget(rdfRes, attributeRes, true).QueryInterface(Components.interfaces.nsIRDFLiteral)
//		jsdump("persist value {0} = {1} ".format(attribute, screenX.Value));
		element.setAttribute(attribute, attributeValue.Value);
//		window.moveTo(screenX.Value, screenY.Value);
	}
}

function XuluTreeCommandController(treeView) {
	this.treeView = treeView;
/*	var commandSet = document.createElement("commandset");
	commandSet.setAttribute("id", "testCommandSet");
	commandSet.setAttribute("commandupdater", "true");
	commandSet.setAttribute("events", "focus");
	commandSet.setAttribute("oncommandupdate", "jsdump('testCommand')");
	this.treeView.tree.appendChild(commandSet); */
	var contextMenu = document.createElement("popup");
	var menuId = this.treeView.tree.getAttribute("id") + "-context-menu";
	contextMenu.setAttribute("id" , menuId);
	this.copyCommand = this.createCommand(menuId + "-copy-command", "Copy");
	var copyMenuItem = this.addMenuItem(menuId + "-copy-menuitem", this.copyCommand, contextMenu);
	this.groupByCommand = this.createCommand(menuId + "-groupBy-command", "Group by");;
	var groupByMenuItem = this.addMenuItem(menuId + "-groupBy-menuitem", this.groupByCommand, contextMenu);
	this.removeGroupingCommand = this.createCommand(menuId + "-removeGrouping-command", "Remove Grouping");
	this.addMenuItem(menuId + "removeGrouping-menuitem", this.removeGroupingCommand, contextMenu);
	this.treeView.tree.appendChild(contextMenu);
	this.treeView.tree.contextMenu = menuId;
	contextMenu.addEventListener("command", this, false);
	contextMenu.addEventListener("popupshowing", this, false);
	this.treeView.tree.addEventListener("keypress", this, false);
	this.treeView.tree.controllers.appendController(this);
}

XuluTreeCommandController.prototype.createCommand = function(id, label){
	var command = document.createElement("command");
	command.setAttribute("id", id);
	command.setAttribute("label", label);
	this.treeView.tree.appendChild(command);
	command.addEventListener("command", this, false);
	return command;
}

XuluTreeCommandController.prototype.addMenuItem = function(id, command, contextMenu){
	var menuItem = document.createElement("menuitem");
	menuItem.setAttribute("id", id);
	menuItem.setAttribute("command", command.id);
	contextMenu.appendChild(menuItem);
	return menuItem;
}

XuluTreeCommandController.prototype.onKeyPress = function(e) {
	jsdump("Key press");
	if (e.ctrlKey) {
		switch (e.which) {
			case 97 : // A
				this.treeView.selection.selectAll();
			case 99 : // C
//				this.copySelected();  // We will deal with this case through cmd_copy
				break;
			default : jsdump(e.which);
		}
	}	
}

XuluTreeCommandController.prototype.onPopupShowing = function(e){
	var target = document.popupNode;
	this.removeGroupingCommand.setAttribute("disabled", this.treeView.groupByColumnName == null, "true", "false");
	switch (target.nodeName) {
		case "treecol" :
			this.groupByCommand.setAttribute("label", "Group by " + target.getAttribute("label"));
			if (this.treeView.groupByColumnName == target.id) {
				this.groupByCommand.setAttribute("disabled", "true");						
			} else {
				this.groupByCommand.setAttribute("disabled", "false");
			}
			break;
		default :
			this.groupByCommand.setAttribute("disabled", "true");
	}
}

XuluTreeCommandController.prototype.onCommand = function(e){
	var target = document.popupNode;
	switch (e.target.id) {
		case this.copyCommand.id :
			this.copySelected();
			break;
		case this.groupByCommand.id :
			if (target.nodeName == "treecol") {
				this.treeView.groupByColumnName = target.id;
			}
			break;
		case this.removeGroupingCommand.id :
			this.treeView.groupByColumnName = null;
			break;
	}
}

// START: nsIDOMEventListener interface
XuluTreeCommandController.prototype.handleEvent = function(e) {
	switch (e.type) {
		case "popupshowing" :
			this.onPopupShowing(e);
			break;
		case "command" :
			this.onCommand(e);
			break;
		case "keypress" :
			this.onKeyPress(e);
			break;
	}
}
// END: nsIDOMEventListener interface

XuluTreeCommandController.prototype.copySelected = function() {
	var start = new Object();
	var end = new Object();
	var numRanges = this.treeView.selection.getRangeCount();
	if (!numRanges) {
		return;
	}
	
	var selectionCount = this.treeView.selection.count
	if (selectionCount > 100) {
		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
		var result = prompts.confirm(window, "Copy {0} rows?".format(selectionCount), "Are you sure you want to copy {0} rows to the clipboard?".format(selectionCount));
		if (!result) {
			return;
		}
	}
	var orderedColumns = [];
	var treeColsBoxObject = this.treeView.treeCols.boxObject;
	var currentChild = this.treeView.treeCols.boxObject.firstChild;
	while (currentChild) {
		if (currentChild.nodeName == "treecol" && !currentChild.hidden) {
			orderedColumns.push(currentChild);
		}
		currentChild = currentChild.nextSibling;
	}
	var columnCount = orderedColumns.length;
	
	var selectedText = "";
	
	for (var t = 0; t < numRanges; t++){
		this.treeView.selection.getRangeAt(t,start,end);
		for (var v = start.value; v <= end.value; v++){
			var rowText = "";
			for (var i=0; i < columnCount; i++) {
				if (i>0) {
					rowText +="\t";
				}
				rowText += this.treeView.getCellText(v, orderedColumns[i]);
			}
			if (selectedText) {
				selectedText += "\n";
			}
			selectedText += rowText;
		}
	}
	gClipboardHelper.copyString(selectedText);	
}

// START: nsIController interface
XuluTreeCommandController.prototype.supportsCommand = function(cmd){
//		jsdump("Supports? " + cmd);
	switch (cmd) {
		case "cmd_copy" : return true;
	}
}

XuluTreeCommandController.prototype.isCommandEnabled = function(cmd){
	switch (cmd) {
		case "cmd_copy" :
			return true;
	}
}

XuluTreeCommandController.prototype.doCommand = function(cmd){
	switch (cmd) {
		case "cmd_copy" :
			this.copySelected();
	}
}

XuluTreeCommandController.prototype.onEvent = function(evt){
	jsdump(evt.target.id)
}
// END: nsIController interface

function XuluTreeView(tree, dataSource, treeMap, DBConn, tempTableName) {
	this.tree = tree;
	this._oldRowCount = 0;
	this.filter = {};
    this.objects = [];
	this.cache = {};
	this.orderClause = "";
	this.orderColumnIndex = 0;
    this.orderDirection = 1;
	this.tempTableName = tempTableName;
	this.dataSourceListener =  new XuluTreeDataSourceListener(this);
	this.tempDataSource = null;
	if (DBConn) {
		this.DBConn = DBConn;
	} else if (this.dataSource) {
//		this.DBConn = this.dataSource.newConnection();
	}
	this.dataSource = dataSource;
	this.treeMap = treeMap;
	this.groupByColumnName = null;
	this._groupByResults = null;
	
//	this.treeCols = document.evaluate('./treecols', this.tree, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null ).singleNodeValue;
	this.treeCols = this.tree.firstChild;
	for (var i = this.treeCols.childNodes.length - 1; i >=0; i--) {
		this.treeCols.removeChild(this.treeCols.childNodes[i]);
	}
	for (var i = 0, l = this.treeMap.columns.length; i < l; i++) {
		var treeCol = document.createElement("treecol");
		this.treeMap.columns[i].DOM = treeCol;
		this.treeMap.columns[this.treeMap.columns[i].columnName] = this.treeMap.columns[i];
		treeCol.setAttribute("id", this.treeMap.columns[i].columnName);
		treeCol.setAttribute("label", this.treeMap.columns[i].label);
		treeCol.setAttribute("flex", "1");
		treeCol.setAttribute("persist", "width hidden sortActive sortDirection");
		if (this.treeMap.columns[i].type == "currency") {
			treeCol.setAttribute("style", "text-align: right");		
		}
		setPersistValue(treeCol, "hidden");
		setPersistValue(treeCol, "width");
		setPersistValue(treeCol, "sortActive");
		setPersistValue(treeCol, "sortDirection");
		var sortDirection = treeCol.getAttribute("sortDirection")
		if (sortDirection) {
			if (sortDirection != "natural") {
				this.orderColumnIndex = i;
				this.orderDirection = sortDirection == "ascending"
			}
		}
//		treeCol.setAttribute("persist", "sortActive sortDirection width hidden");
		var splitter = document.createElement("splitter");
		splitter.setAttribute("class", "tree-splitter");
		this.treeCols.appendChild(treeCol);
		this.treeCols.appendChild(splitter);
	}
	
	this.commandController = new XuluTreeCommandController(this);
};

XuluTreeView.prototype.terminate = function() {
	this.dataSource = null;
	this.DBConn = null;
	this.tree = null;
}

var XuluTreeDataSourceListener = function(treeView) {
	this.treeView = treeView;

	this.insertEvent = function() {
		this.treebox.rowCountChanged(this._oldRowCount - 1, this.rowCount - this._oldRowCount);
		this._oldRowCount = this.rowCount;
	},
	
	this.dataStoreEvent = function(event, DBConn) {
		if (!DBConn.transactionInProgress) {
			this.treeView.invalidate();
		}
	}
	
}

XuluTreeView.prototype.invalidate = function(invalidateSQL) {
	this.cache = {};
	if (invalidateSQL && this.dataSource && this.treeMap) {
		var member = this.treeMap.columns[this.orderColumnIndex].member
		var fieldMap = this.dataSource.dataMap.fieldMappings[member];
		if (fieldMap) {
			var orderByName = fieldMap.fieldName;
			if (fieldMap.type == "string") {
				orderByName = "upper("+ orderByName + ")"
			}
		}
				
		if (this.groupByColumnName != null) {
			var member = this.treeMap.columns[this.groupByColumnName].member
			var fieldMap = this.dataSource.dataMap.fieldMappings[member];
			if (fieldMap) {
				var fieldName = fieldMap.fieldName;
				orderByName = fieldName + ", " + orderByName;
			}
		}
		
		this.orderClause = "ORDER BY " + orderByName + (this.orderDirection ? "": " DESC")
		if (this.tempTableName) {
			var options = {
				whereClause: this._filter.whereClause,
				whereValues: this._filter.whereValues,
				orderClause: this.orderClause
			}
			this.tempDataSource = this.dataSource.createTempDataStore(this.tempTableName, options, this.DBConn);
		} else {
			this.tempDataSource = null;
		}

//		jsdump ("Order clause: " + this.orderClause);
	}
	if (this.treebox) {
		this.treebox.rowCountChanged(this._oldRowCount - 1, this.rowCount - this._oldRowCount);
		this._oldRowCount = this.rowCount;
		this.treebox.invalidate();
	}
	
	if (this.selection) {
		this.selection.clearSelection();
	}
}

XuluTreeView.prototype.getGroupResult = function() {
	if (this.groupByColumnName != null) {
		if (this.dataSource) {
			var member = this.treeMap.columns[this.groupByColumnName].member
			var fieldMap = this.dataSource.dataMap.fieldMappings[member];
			if (fieldMap) {
				var fieldName = fieldMap.fieldName;
				var whereClause = "";
				if (this.filter && this.filter.whereClause) {
					whereClause = this.filter.whereClause;
				}
				var groupSQL = "SELECT {0}, COUNT(*) FROM {1} {2} GROUP BY {3}".format(fieldName, this.dataSource.dataMap.tableName, whereClause, fieldName);
//				jsdump ("Group SQL: " + groupSQL);
//				jsdump(this.filter.whereValues[1].value);
				this._groupByResults = this.dataSource.xuluData.executeStatement(groupSQL, this.filter.whereValues, this.DBConn);
				this.calcGroupRows();
			}
		}
	}
	
}
XuluTreeView.prototype.calcGroupRows = function() {
	if (this._groupByResults) {
		this._groupRows = [];
		var sqlRow = 0;
		for (var i=0, l = this._groupByResults.length; i < l; i++) {
			var groupResult = this._groupByResults[i];
			var childCount = parseInt(this._groupByResults[i][1]);
			this._groupRows.push({
				isContainer : true,
				isContainerOpen :  groupResult.open ? true : false,
				isContainerEmpty : false,
				getLevel : 0,
				getParentIndex : -1,
				nextSibling : groupResult.open ? this._groupRows.length + childCount + 1 : this._groupRows.length + 1,
				resultIndex : i
			});
			if (groupResult.open) {
				var parentIndex = this._groupRows.length - 1;
				for (var j=0; j < childCount; j++) {
					this._groupRows.push({
						isContainer : false,
						isContainerOpen :  false,
						isContainerEmpty : false,
						getLevel : 1,
						getParentIndex : parentIndex,
						nextSibling : parentIndex + j + 2,
						sqlRow : sqlRow++
					});				
				}				
			} else {
				sqlRow += childCount;
			}
		}
	} else {
		this._groupRows = null;
	}
};

XuluTreeView.prototype.__defineGetter__("dataSource", function() {
	return this._dataSource;
});

XuluTreeView.prototype.__defineSetter__("dataSource", function(dataSource) {
	this._dataSource = dataSource;
	if (dataSource) {
		this._dataSource.addEventListener("", this.dataSourceListener);
		this.getGroupResult();
		this.invalidate(true);
	}
});

XuluTreeView.prototype.__defineGetter__("treeMap", function() {
	return this._treeMap;
});

XuluTreeView.prototype.__defineSetter__("treeMap", function(treeMap) {
	this._treeMap = treeMap;
	if (treeMap) {
		this.invalidate(true);
	}
});

XuluTreeView.prototype.__defineGetter__("filter", function() {
	return this._filter;
});

XuluTreeView.prototype.__defineSetter__("filter", function(filter) {
	this._filter = filter;
	this.getGroupResult();
	this.invalidate(true);
});

XuluTreeView.prototype.__defineGetter__("rowCount", function() {
//	jsdump("Get row count");
	if (this._dataSource) {
		if (this._groupByResults) {
			return this._groupRows.length;
		} else {
			if (this.tempDataSource) {
				return this.tempDataSource.getCount(null, this.DBConn);
			} else {
				return this._dataSource.getCount(this._filter, this.DBConn);
			}
		}
	} else {
		return 0;
	}
	return this.objects.length;
});

XuluTreeView.prototype.groupBy = function(e) {
	var target = document.popupNode;
	if (target.nodeName == "treecol") {
		this.groupByColumnName = target.id;
	}
	jsdump(e);
};

XuluTreeView.prototype.__defineGetter__("groupByColumnName", function() {
	return this._groupByColumnName;
});

XuluTreeView.prototype.__defineSetter__("groupByColumnName", function(columnName) {
	if (columnName == this._groupByColumnName) {
		return;
	}
	if (this._groupByColumnName != null) {
		this.treeMap.columns[this.groupByColumnName].DOM.setAttribute("primary", "false");
	}
	this._groupByColumnName = columnName;
	if (columnName) {
		this.getGroupResult();
		var dom = this.treeMap.columns[this.groupByColumnName].DOM;
//		var firstDOM = this.treeMap.columns[0].DOM;
//		this.treeCols.removeChild(dom);
//		this.treeCols.insertBefore(dom, firstDOM);

		dom.setAttribute("primary", "true");
	} else {
		this._groupByResults = null;
	}
	this.invalidate(true);
});

XuluTreeView.prototype.getCellText = function(row,column) {
// 	jsdump ("Row {0}, Column {1}".format(row, column));
	var sqlRow = row;
	if (this._groupByResults) {
		if (this._groupRows[row].isContainer) {
			//		jsdump("Get Cell Text: " + row);
			if (column.id == this.groupByColumnName) {
				return this._groupByResults[this._groupRows[row].resultIndex][0];
			} else {
				return "";
			}
		}
		else {
			var sqlRow = this._groupRows[row].sqlRow;
			var member = this.treeMap.columns[this.groupByColumnName].member
			var fieldMap = this.dataSource.dataMap.fieldMappings[member];
			if (fieldMap) {
				var fieldName = fieldMap.fieldName;
			}
		}
	} else {
		var sqlRow = row;
	}
	if (this.cache[sqlRow]) {
		var object = this.cache[sqlRow];
	}
	else {
		if (this.tempDataSource) {
			var options = {
				limitClause: " LIMIT 1 OFFSET " + sqlRow,
			}
			var object = this.tempDataSource.loadObjects(options, this.DBConn)[0];
		} else {
			var options = {
				whereClause: this._filter.whereClause,
				whereValues: this._filter.whereValues,
				orderClause: this.orderClause,
				limitClause: " LIMIT 1 OFFSET " + sqlRow,
			}
			var object = this._dataSource.loadObjects(options, this.DBConn)[0];
		}
		this.cache[sqlRow] = object;
	}
	if (object) {
		var cellValue = getMember(object, this.treeMap.columns[column.id].member);
		switch (this.treeMap.columns[column.index].type) {
			case "currency" :
				cellValue = formatNumberAsString(2, cellValue)
				break;
			case "image" :
				cellValue = null;
				break;				
			default :
				if (this.treeMap.columns[column.index].cellFormatter) {
					cellValue = this.treeMap.columns[column.index].cellFormatter(cellValue);
				}
				break;
		}
		return cellValue;
	}
};

XuluTreeView.prototype.setTree = function(treebox) {
	this.treebox = treebox;
	if (treebox) {
		this.invalidate();
	}
};

XuluTreeView.prototype.isContainer = function(row){ 
	if (this._groupByResults) {
//		jsdump("Is Container: " + row);
		return this._groupRows[row].isContainer;
	} else {
		return false;	
	}
//	return row == 0; 
}

XuluTreeView.prototype.isContainerOpen = function(row){
	if (this._groupByResults) {
//		jsdump("Is Container Open: " + row);
		return this._groupRows[row].isContainerOpen;
	} else {
		return false;	
	}
}

XuluTreeView.prototype.isContainerEmpty = function(row) {
	if (this._groupByResults) {
//		jsdump("Is Container Empty: " + row);
		return this._groupRows[row].isContainerEmpty;
	} else {
		return false;	
	}
},

XuluTreeView.prototype.isSeparator = function(row) {
	return false;
	return row == 2
}

XuluTreeView.prototype.isSorted = function() {
	return true;
}

XuluTreeView.prototype.getLevel = function(row) {
//	jsdump("Get Level: " + row);
	if (this._groupByResults) {
		return this._groupRows[row].getLevel;
		if (this.getParentIndex(row) == -1) {
			return 0;
		} else {
			return 1;
		}
	} else {
		return 0;
	}
	return 0;
	return row == 0 ? 0 : 1;
},

XuluTreeView.prototype.getParentIndex = function(row) {
	if (this._groupByResults) {
//		jsdump("Is Container Empty: " + row);
		return this._groupRows[row].getParentIndex;
		var val = row / 2
		if (Math.floor(val) == Math.ceil(val)) {
			return -1
		} else {
			return row - 1;
		}
		return false;
	} else {
		return 1;	
	}
	return -1;
	return row == 0 ? -1 : 0;
},

XuluTreeView.prototype.hasNextSibling = function(row, afterIndex) {
	if (this._groupByResults) {
//		jsdump("has Next Sibling: " + row);
		return this._groupRows[row].nextSibling == afterIndex - 1;
		return row == afterIndex;
	} else {
		return row == afterIndex;	
	}
	return row == afterIndex;
	return row == 0 ? false : row == afterIndex;
}	

XuluTreeView.prototype.getImageSrc = function(row,col) {
	try {
		if (this.treeMap.columns[col.index].type == "image") {
			var rowObject = this.getRowObject(row);
			var imageSrc = getMember(rowObject, this.treeMap.columns[col.id].member);
			if (this.treeMap.columns[col.index].cellFormatter) {
				imageSrc = this.treeMap.columns[col.index].cellFormatter(imageSrc);
			}
		} else {
			var imageSrc = null;				
		}
		return imageSrc;		
	} catch (e) {
		XuluErrorHandler.handleError("Tree: getImageSrc", e);
	}
}

XuluTreeView.prototype.getRowProperties = function(row,props) {
	
	if (this.isContainer(row)) {
		props.AppendElement(gAtomService.getAtom("isContainer"));
	}
}

XuluTreeView.prototype.getCellProperties = function(row,col,props) {
	if (this.isContainer(row)) {
		props.AppendElement(gAtomService.getAtom("isContainer"));
	}
	if (this.treeMap.columns[col.index].getStyleProperties) {
		var propertyArray = this.treeMap.columns[col.index].getStyleProperties(row, col, this.getCellText(row, col));
		for (var i=0, l=propertyArray.length; i < l; i++) {
		    props.AppendElement(gAtomService.getAtom(propertyArray[i]));			
		}
	}
}

XuluTreeView.prototype.getColumnProperties = function(colid,col,props) {
	
}

XuluTreeView.prototype.cycleHeader = function (col) {
    if (col.index == this.orderColumnIndex) {
		this.orderDirection = !this.orderDirection;
	} else {
		var oldColumn = document.getElementById(this.treeMap.columns[this.orderColumnIndex].columnName);
		oldColumn.setAttribute("sortDirection", "natural");
		this.orderColumnIndex = col.index;
	}
	col.element.setAttribute("sortDirection", this.orderDirection ? "ascending" : "descending");
	this.invalidate(true);
}

XuluTreeView.prototype.toggleOpenState = function(row){
	if (this._groupByResults) {
		if (this._groupRows[row].isContainer) {
			//		jsdump("Get Cell Text: " + row);
			this._groupByResults[this._groupRows[row].resultIndex].open = !this._groupByResults[this._groupRows[row].resultIndex].open;
			this.calcGroupRows();
			this.invalidate();
		}
	}
}